@@ -44,7 +44,7 @@ aterm_appl find_if(Term t, MatchPredicate match)
   {
     detail::find_if_impl< typename boost::add_reference< MatchPredicate >::type >(aterm_traits<Term>::term(t), match);
   }
-  catch (detail::found_term_exception e)
+  catch (detail::found_term_exception& e)
   {
     return e.t;
   }
@@ -65,7 +65,7 @@ aterm_appl partial_find_if(Term t, MatchPredicate match, StopPredicate stop)
   {
     detail::partial_find_if_impl< typename boost::add_reference< MatchPredicate >::type >(aterm_traits<Term>::term(t), match, stop);
   }
-  catch (detail::found_term_exception e)
+  catch (detail::found_term_exception& e)
   {
     return e.t;
   }
