@@ -12,6 +12,7 @@
 #ifndef MCRL2_BES_PG_PARSE_H
 #define MCRL2_BES_PG_PARSE_H
 
+#include <fstream>
 #include <cstdio>
 #include <cctype>
 #include <map>
@@ -27,6 +28,7 @@ namespace bes
 
 typedef unsigned long long identifier_t;
 typedef unsigned short priority_t;
+
 typedef bool owner_t;
 
 struct node_t
@@ -45,7 +47,8 @@ struct node_t
 // Build a formula from the strings in v. if p = 0, than a disjunction is built,
 // otherwise the result is a conjunction.
 // Prefix is added to each of the identifiers in v.
-boolean_expression formula(std::set<identifier_t> const& v, priority_t p, std::string prefix = "X")
+inline
+boolean_expression formula(std::set<identifier_t> const& v, const owner_t owner, std::string prefix = "X")
 {
   atermpp::set<boolean_expression> v_prefixed;
   for (std::set<identifier_t>::const_iterator i = v.begin(); i != v.end(); ++i)
@@ -55,7 +58,7 @@ boolean_expression formula(std::set<identifier_t> const& v, priority_t p, std::s
     v_prefixed.insert(boolean_variable(id.str()));
   }
 
-  if (p == 0)
+  if (owner == 0)
   {
     return join_or(v_prefixed.begin(), v_prefixed.end());
   }
@@ -65,7 +68,7 @@ boolean_expression formula(std::set<identifier_t> const& v, priority_t p, std::s
   }
 }
 
-/// \brief Reads a parity games from an input stream, and stores it as a BES.
+/// \brief Reads a parity game from an input stream, and stores it as a BES.
 /// \param from An input stream
 /// \param b A boolean equation system
 /// \return The input stream
@@ -82,7 +85,7 @@ boolean_expression formula(std::set<identifier_t> const& v, priority_t p, std::s
 // <identifier> and <priority>, <priority> and <owner>, <owner> and <identifier>
 template <typename Container>
 inline
-void parse_pgsolver(std::istream& from, boolean_equation_system<Container>& b)
+void parse_pgsolver(std::istream& from, boolean_equation_system<Container>& b, bool maxpg = true)
 {
   while (!isalnum(from.peek()))
   {
@@ -98,7 +101,7 @@ void parse_pgsolver(std::istream& from, boolean_equation_system<Container>& b)
   // Parse node specifications (store in map)
   std::map<identifier_t, node_t> game;
   bool init = false;
-  identifier_t initial_node;
+  identifier_t initial_node = 0;
 
   while (isalnum(from.peek()))
   {
@@ -162,16 +165,46 @@ void parse_pgsolver(std::istream& from, boolean_equation_system<Container>& b)
   }
 
   atermpp::vector<boolean_equation> eqns;
-  for (std::map<priority_t, atermpp::set<boolean_equation> >::const_iterator i = blocks.begin(); i != blocks.end(); ++i)
+  if(maxpg)
   {
-    eqns.insert(eqns.end(), i->second.begin(), i->second.end());
+    for (std::map<priority_t, atermpp::set<boolean_equation> >::reverse_iterator i = blocks.rbegin(); i != blocks.rend(); ++i)
+    {
+      eqns.insert(eqns.end(), i->second.begin(), i->second.end());
+    }
+  }
+  else
+  {
+    for (std::map<priority_t, atermpp::set<boolean_equation> >::const_iterator i = blocks.begin(); i != blocks.end(); ++i)
+    {
+      eqns.insert(eqns.end(), i->second.begin(), i->second.end());
+    }
   }
 
   b.equations() = eqns;
   std::stringstream init_id;
   init_id << initial_node;
   b.initial_state() = boolean_variable("X" + init_id.str());
+}
 
+/// \brief Parse parity game in PGSolver format from filename, and store the
+///        resulting BES in b.
+template <typename Container>
+inline void parse_pgsolver(const std::string& filename, boolean_equation_system<Container>& b, bool maxpg = true)
+{
+  if(filename == "-" || filename.empty())
+  {
+    parse_pgsolver(std::cin, b, maxpg);
+  }
+  else
+  {
+    std::ifstream f;
+    f.open(filename.c_str());
+    if(!f)
+    {
+      throw mcrl2::runtime_error("cannot open file " + filename + " for reading");
+    }
+    parse_pgsolver(f, b, maxpg);
+  }
 }
 
 } // namespace bes
