@@ -16,9 +16,9 @@
 #include <cerrno>
 #include <string>
 #include <cstring>
+#include "mcrl2/aterm/aterm2.h"
 #include "mcrl2/exception.h"
 #include "mcrl2/atermpp/aterm_appl.h"
-#include "aterm2.h"
 
 namespace mcrl2
 {
@@ -96,7 +96,7 @@ void save_aterm(ATerm term, const std::string& filename, bool binary = true)
     throw mcrl2::runtime_error("could not open output file '" + filename + "' for writing (" + err_msg + ")");
   }
   //write specification to stream
-  ATbool result;
+  bool result;
   if (binary)
   {
     result = ATwriteToSAFFile(term, stream);
@@ -109,7 +109,7 @@ void save_aterm(ATerm term, const std::string& filename, bool binary = true)
   {
     fclose(stream);
   }
-  if (result == ATfalse)
+  if (result == false)
   {
     throw mcrl2::runtime_error("could not write ATerm to " + ((stream == stdout)?"stdout":("'" + filename + "'")));
   }
