@@ -15,6 +15,7 @@
 #include "mcrl2/aterm/aterm2.h"
 #include "mcrl2/data/data_specification.h"
 #include "mcrl2/data/rewriter.h"
+#include "mcrl2/data/detail/prover/solver_type.h"
 #include "mcrl2/data/detail/prover.h"
 #include "mcrl2/data/detail/prover/bdd_simplifier.h"
 #include "mcrl2/data/detail/prover/bdd_path_eliminator.h"
@@ -43,7 +44,7 @@ namespace detail
  * prover's rewriter with the data equations in internal mCRL2 format
  * contained in the LPS passed as parameter a_lps and the rewrite
  * strategy passed as parameter a_rewrite_strategy. The parameter
- * a_rewrite_strategy can be set to either 
+ * a_rewrite_strategy can be set to either
  * GS_REWR_JITTY or GS_REWR_JITTYC. To limit the
  * number of seconds spent on proving a single formula, a time limit
  * can be set. If the time limit is set to 0, no time limit will be
@@ -82,6 +83,10 @@ namespace detail
 
 class BDD_Prover: public Prover
 {
+  public:
+    typedef Prover::substitution_type substitution_type;
+    typedef Prover::internal_substitution_type internal_substitution_type;
+
   private:
 
     /// \brief Flag indicating whether or not the result of the comparison between the first two arguments
@@ -101,11 +106,11 @@ class BDD_Prover: public Prover
 
     /// \brief A hashtable that maps formulas to BDDs.
     /// \brief If the BDD of a formula is unknown, it maps this formula to 0.
-    ATermTable f_formula_to_bdd;
+    atermpp::map < atermpp::aterm_appl, atermpp::aterm_appl > f_formula_to_bdd;
 
     /// \brief A hashtable that maps formulas to the smallest guard occuring in those formulas.
     /// \brief If the smallest guard of a formula is unknown, it maps this formula to 0.
-    ATermTable f_smallest;
+    atermpp::map < atermpp::aterm_appl, atermpp::aterm_appl > f_smallest;
 
     /// \brief Class that provides information about the structure of BDDs.
     BDD_Info f_bdd_info;
@@ -119,39 +124,36 @@ class BDD_Prover: public Prover
     /// \brief Constructs the EQ-BDD corresponding to the formula Prover::f_formula.
     void build_bdd()
     {
-      f_formula_to_bdd = ATtableCreate(60000, 25);
-      f_smallest = ATtableCreate(2000, 50);
       f_deadline = time(0) + f_time_limit;
 
-      ATerm v_previous_1 = 0;
-      ATerm v_previous_2 = 0;
+      atermpp::aterm_appl v_previous_1;
+      atermpp::aterm_appl v_previous_2;
 
-      mCRL2log(debug) << "Formula: " << core::pp( f_formula) << std::endl;
+      mCRL2log(log::debug) << "Formula: " << data::pp( f_formula) << std::endl;
 
       f_internal_bdd = m_rewriter->toRewriteFormat(f_formula);
-      f_internal_bdd = m_rewriter->rewriteInternal(f_internal_bdd);
-      f_internal_bdd = f_manipulator->orient(f_internal_bdd);
 
-      mCRL2log(debug) << "Formula rewritten and oriented: " << core::pp( m_rewriter->fromRewriteFormat(f_internal_bdd)) << std::endl;
+      f_internal_bdd = m_rewriter->rewrite_internal(f_internal_bdd,bdd_sigma);
+      f_internal_bdd = f_manipulator.orient(f_internal_bdd);
+
+      mCRL2log(log::debug) << "Formula rewritten and oriented: " << data::pp( m_rewriter->fromRewriteFormat(f_internal_bdd)) << std::endl;
 
       while (v_previous_1 != f_internal_bdd && v_previous_2 != f_internal_bdd)
       {
         v_previous_2 = v_previous_1;
         v_previous_1 = f_internal_bdd;
         f_internal_bdd = bdd_down(f_internal_bdd);
-        mCRL2log(debug) << "End of iteration." << std::endl;
-        mCRL2log(debug) << "Intermediate BDD: " << core::pp( m_rewriter->fromRewriteFormat(f_internal_bdd)) << std::endl;
+        mCRL2log(log::debug) << "End of iteration." << std::endl;
+        mCRL2log(log::debug) << "Intermediate BDD: " << data::pp( m_rewriter->fromRewriteFormat(f_internal_bdd)) << std::endl;
       }
 
       f_bdd = m_rewriter->fromRewriteFormat(f_internal_bdd);
-      mCRL2log(debug) << "Resulting BDD: " << core::pp( f_bdd) << std::endl;
+      mCRL2log(log::debug) << "Resulting BDD: " << data::pp( f_bdd) << std::endl;
 
-      ATtableDestroy(f_formula_to_bdd);
-      ATtableDestroy(f_smallest);
     }
 
     /// \brief Creates the EQ-BDD corresponding to the formula a_formula.
-    ATerm bdd_down(ATerm a_formula)
+    atermpp::aterm_appl bdd_down(atermpp::aterm_appl a_formula)
     {
       std::string indent;
 
@@ -159,59 +161,59 @@ class BDD_Prover: public Prover
     }
 
     /// \brief Creates the EQ-BDD corresponding to the formula a_formula.
-    ATerm bdd_down(ATerm a_formula, std::string& a_indent)
+    atermpp::aterm_appl bdd_down(atermpp::aterm_appl a_formula, std::string& a_indent)
     {
       a_indent.append("  ");
 
       if (f_time_limit != 0 && (f_deadline - time(0)) <= 0)
       {
-        mCRL2log(debug) << "The time limit has passed." << std::endl;
+        mCRL2log(log::debug) << "The time limit has passed." << std::endl;
         return a_formula;
       }
 
-      if (f_info->is_true(a_formula))
+      if (f_info.is_true(a_formula))
       {
         return a_formula;
       }
-      if (f_info->is_false(a_formula))
+      if (f_info.is_false(a_formula))
       {
         return a_formula;
       }
 
-      ATerm v_bdd = ATtableGet(f_formula_to_bdd, a_formula);
-      if (v_bdd)
+      const atermpp::map < atermpp::aterm_appl, atermpp::aterm_appl >::const_iterator i = f_formula_to_bdd.find(a_formula);
+      if (i!=f_formula_to_bdd.end()) // found
       {
-        return v_bdd;
+        return i->second;
       }
 
-      ATerm v_guard = smallest(a_formula);
+      atermpp::aterm_appl v_guard = smallest(a_formula);
       if (!v_guard)
       {
         return a_formula;
       }
       else
       {
-        mCRL2log(debug) << a_indent << "Smallest guard: " << core::pp(m_rewriter->fromRewriteFormat(v_guard)) << std::endl;
+        mCRL2log(log::debug) << a_indent << "Smallest guard: " << data::pp(m_rewriter->fromRewriteFormat(v_guard)) << std::endl;
       }
 
-      ATerm v_term1, v_term2;
+      atermpp::aterm_appl v_term1, v_term2;
 
-      v_term1 = f_manipulator->set_true(a_formula, v_guard);
-      v_term1 = m_rewriter->rewriteInternal(v_term1);
-      v_term1 = f_manipulator->orient(v_term1);
-      mCRL2log(debug) << a_indent << "True-branch after rewriting and orienting: " << core::pp(m_rewriter->fromRewriteFormat(v_term1)) << std::endl;
+      v_term1 = f_manipulator.set_true(a_formula, v_guard);
+      v_term1 = m_rewriter->rewrite_internal(v_term1,bdd_sigma);
+      v_term1 = f_manipulator.orient(v_term1);
+      mCRL2log(log::debug) << a_indent << "True-branch after rewriting and orienting: " << data::pp(m_rewriter->fromRewriteFormat(v_term1)) << std::endl;
       v_term1 = bdd_down(v_term1, a_indent);
-      mCRL2log(debug) << a_indent << "BDD of the true-branch: " << core::pp(m_rewriter->fromRewriteFormat(v_term1)) << std::endl;
+      mCRL2log(log::debug) << a_indent << "BDD of the true-branch: " << data::pp(m_rewriter->fromRewriteFormat(v_term1)) << std::endl;
 
-      v_term2 = f_manipulator->set_false(a_formula, v_guard);
-      v_term2 = m_rewriter->rewriteInternal(v_term2);
-      v_term2 = f_manipulator->orient(v_term2);
-      mCRL2log(debug) << a_indent << "False-branch after rewriting and orienting: " << core::pp(m_rewriter->fromRewriteFormat(v_term2)) << std::endl;
+      v_term2 = f_manipulator.set_false(a_formula, v_guard);
+      v_term2 = m_rewriter->rewrite_internal(atermpp::aterm_appl(v_term2),bdd_sigma);
+      v_term2 = f_manipulator.orient(v_term2);
+      mCRL2log(log::debug) << a_indent << "False-branch after rewriting and orienting: " << data::pp(m_rewriter->fromRewriteFormat(v_term2)) << std::endl;
       v_term2 = bdd_down(v_term2, a_indent);
-      mCRL2log(debug) << a_indent << "BDD of the false-branch: " << core::pp(m_rewriter->fromRewriteFormat(v_term2)) << std::endl;
+      mCRL2log(log::debug) << a_indent << "BDD of the false-branch: " << data::pp(m_rewriter->fromRewriteFormat(v_term2)) << std::endl;
 
-      v_bdd = f_manipulator->make_reduced_if_then_else(v_guard, v_term1, v_term2);
-      ATtablePut(f_formula_to_bdd, a_formula, v_bdd);
+      atermpp::aterm_appl v_bdd = f_manipulator.make_reduced_if_then_else(v_guard, v_term1, v_term2);
+      f_formula_to_bdd[a_formula]=v_bdd;
 
       a_indent.erase(a_indent.size() - 2);
 
@@ -226,10 +228,10 @@ class BDD_Prover: public Prover
       v_new_time_limit = f_deadline - time(0);
       if (v_new_time_limit > 0 || f_time_limit == 0)
       {
-        mCRL2log(debug) << "Simplifying the BDD:" << std::endl;
+        mCRL2log(log::debug) << "Simplifying the BDD:" << std::endl;
         f_bdd_simplifier->set_time_limit((std::max)(v_new_time_limit, time(0)));
         f_bdd = f_bdd_simplifier->simplify(f_bdd);
-        mCRL2log(debug) << "Resulting BDD: " << core::pp( f_bdd) << std::endl;
+        mCRL2log(log::debug) << "Resulting BDD: " << data::pp( f_bdd) << std::endl;
       }
     }
 
@@ -240,14 +242,14 @@ class BDD_Prover: public Prover
       {
         build_bdd();
         eliminate_paths();
-        ATermAppl v_original_formula = f_formula;
-        ATermAppl v_original_bdd = f_bdd;
+        data_expression v_original_formula = f_formula;
+        data_expression v_original_bdd = f_bdd;
         if (f_apply_induction && !(f_bdd_info.is_true(f_bdd) || f_bdd_info.is_false(f_bdd)))
         {
           f_induction.initialize(v_original_formula);
           while (f_induction.can_apply_induction() && !f_bdd_info.is_true(f_bdd))
           {
-            mCRL2log(debug) << "Applying induction." << std::endl;
+            mCRL2log(log::debug) << "Applying induction." << std::endl;
             f_formula = f_induction.apply_induction();
             build_bdd();
             eliminate_paths();
@@ -264,7 +266,7 @@ class BDD_Prover: public Prover
             f_induction.initialize(v_original_formula);
             while (f_induction.can_apply_induction() && !f_bdd_info.is_true(f_bdd))
             {
-              mCRL2log(debug) << "Applying induction on the negated formula." << std::endl;
+              mCRL2log(log::debug) << "Applying induction on the negated formula." << std::endl;
               f_formula = f_induction.apply_induction();
               build_bdd();
               eliminate_paths();
@@ -306,46 +308,44 @@ class BDD_Prover: public Prover
     };
 
     /// \brief Returns the smallest guard in the formula a_formula.
-    ATerm smallest(ATerm a_formula)
+    atermpp::aterm_appl smallest(atermpp::aterm_appl a_formula)
     {
-      if (f_info->is_variable(a_formula))
+      if (f_info.is_variable(a_formula))
       {
-        if (f_info->has_type_bool(a_formula))
+        if (f_info.has_type_bool(a_formula))
         {
           return a_formula;
         }
         else
         {
-          return 0;
+          return atermpp::aterm_appl();
         }
       }
-      if (f_info->is_true(a_formula) || f_info->is_false(a_formula))
+      if (f_info.is_true(a_formula) || f_info.is_false(a_formula))
       {
-        return 0;
+        return atermpp::aterm_appl();
       }
 
-      ATerm v_result;
-
-      v_result = ATtableGet(f_smallest, a_formula);
-      if (v_result)
+      const atermpp::map < atermpp::aterm_appl, atermpp::aterm_appl >::const_iterator i = f_smallest.find(a_formula);
+      if (i!=f_smallest.end()) //found
       {
-        return v_result;
+        return i->second;
       }
 
-      size_t i;
+      size_t s;
       size_t v_length;
-      ATerm v_small;
+      atermpp::aterm_appl v_small,v_result;
 
-      v_length = f_info->get_number_of_arguments(a_formula);
+      v_length = f_info.get_number_of_arguments(a_formula);
 
-      for (i = 0; i < v_length; i++)
+      for (s = 0; s < v_length; s++)
       {
-        v_small = smallest(f_info->get_argument(a_formula, i));
+        v_small = smallest(f_info.get_argument(a_formula, s));
         if (v_small)
         {
-          if (v_result)
+          if (v_result!=atermpp::aterm_appl())
           {
-            if (f_info->lpo1(v_result, v_small))
+            if (f_info.lpo1(v_result, v_small))
             {
               v_result = v_small;
             }
@@ -356,35 +356,35 @@ class BDD_Prover: public Prover
           }
         }
       }
-      if (!v_result && f_info->has_type_bool(a_formula))
+      if (!v_result && f_info.has_type_bool(a_formula))
       {
         v_result = a_formula;
       }
-      if (v_result)
+      if (v_result!=atermpp::aterm_appl())
       {
-        ATtablePut(f_smallest, a_formula, v_result);
+        f_smallest[a_formula]=v_result;
       }
 
       return v_result;
     }
 
     /// \brief Returns branch of the BDD a_bdd, depending on the polarity a_polarity.
-    ATermAppl get_branch(ATermAppl a_bdd, bool a_polarity)
+    data_expression get_branch(const data_expression a_bdd, const bool a_polarity)
     {
-      ATermAppl v_result;
+      data_expression v_result;
 
       if (f_bdd_info.is_if_then_else(a_bdd))
       {
-        ATermAppl v_guard = f_bdd_info.get_guard(a_bdd);
-        ATermAppl v_true_branch = f_bdd_info.get_true_branch(a_bdd);
-        ATermAppl v_false_branch = f_bdd_info.get_false_branch(a_bdd);
-        ATermAppl v_branch = get_branch(v_true_branch, a_polarity);
-        if (v_branch == 0)
+        data_expression v_guard = f_bdd_info.get_guard(a_bdd);
+        data_expression v_true_branch = f_bdd_info.get_true_branch(a_bdd);
+        data_expression v_false_branch = f_bdd_info.get_false_branch(a_bdd);
+        data_expression v_branch = get_branch(v_true_branch, a_polarity);
+        if (v_branch == data_expression())
         {
           v_branch = get_branch(v_false_branch, a_polarity);
-          if (v_branch == 0)
+          if (v_branch == data_expression())
           {
-            v_result = 0;
+            v_result = data_expression();
           }
           else
           {
@@ -405,7 +405,7 @@ class BDD_Prover: public Prover
         }
         else
         {
-          v_result = 0;
+          v_result = data_expression();
         }
       }
       return v_result;
@@ -414,10 +414,11 @@ class BDD_Prover: public Prover
   protected:
 
     /// \brief A binary decision diagram in the internal representation of the rewriter.
-    ATerm f_internal_bdd;
+    atermpp::aterm_appl f_internal_bdd;
+    internal_substitution_type bdd_sigma;
 
     /// \brief A binary decision diagram in the internal representation of mCRL2.
-    ATermAppl f_bdd;
+    data_expression f_bdd;
   public:
 
     /// \brief Constructor that initializes the attributes BDD_Prover::f_data_spec, Prover::f_time_limit and
@@ -436,21 +437,22 @@ class BDD_Prover: public Prover
 
     BDD_Prover(
       mcrl2::data::data_specification const& data_spec,
+      const used_data_equation_selector& equations_selector,
       mcrl2::data::rewriter::strategy a_rewrite_strategy = mcrl2::data::rewriter::jitty,
       int a_time_limit = 0,
       bool a_path_eliminator = false,
-      SMT_Solver_Type a_solver_type = solver_type_cvc,
+      smt_solver_type a_solver_type = solver_type_cvc,
       bool a_apply_induction = false)
-      : Prover(data_spec, a_rewrite_strategy, a_time_limit),
+      : Prover(data_spec, equations_selector, a_rewrite_strategy, a_time_limit),
         f_data_spec(data_spec),
         f_induction(data_spec)
     {
       f_reverse = true;
       f_full = true;
       f_apply_induction = a_apply_induction;
-      f_info->set_reverse(f_reverse);
-      f_info->set_full(f_full);
-      mCRL2log(debug) << "Flags:" << std::endl
+      f_info.set_reverse(f_reverse);
+      f_info.set_full(f_full);
+      mCRL2log(log::debug) << "Flags:" << std::endl
                       << "  Reverse: " << bool_to_char_string(f_reverse) << "," << std::endl
                       << "  Full: " << bool_to_char_string(f_full) << "," << std::endl;
       if (a_path_eliminator)
@@ -470,6 +472,18 @@ class BDD_Prover: public Prover
       f_bdd_simplifier = 0;
     }
 
+    /// \brief Set the substitution to be used to construct the BDD
+    void set_substitution(substitution_type &sigma)
+    {
+      bdd_sigma = convert_substitution_to(sigma);
+    }
+
+    /// \brief Set the substitution in internal format to be used to construct the BDD
+    void set_substitution_internal(internal_substitution_type &sigma)
+    {
+      bdd_sigma = sigma;
+    }
+
     /// \brief Indicates whether or not the formula Prover::f_formula is a tautology.
     virtual Answer is_tautology()
     {
@@ -485,54 +499,64 @@ class BDD_Prover: public Prover
     }
 
     /// \brief Returns the BDD BDD_Prover::f_bdd.
-    virtual ATermAppl get_bdd()
+    virtual data_expression get_bdd()
     {
       update_answers();
       return f_bdd;
     }
 
     /// \brief Returns all the guards on a path in the BDD that leads to a leaf labelled "true", if such a leaf exists.
-    virtual ATermAppl get_witness()
+    virtual data_expression get_witness()
     {
-      ATermAppl v_result;
-
       update_answers();
-      if (!(is_contradiction() == answer_yes) && !(is_tautology() == answer_yes))
+      if (is_contradiction() == answer_yes)
       {
-        mCRL2log(debug) << "The formula appears to be satisfiable." << std::endl;
-        v_result = get_branch(f_bdd, true);
+        mCRL2log(log::debug) << "The formula is a contradiction." << std::endl;
+        return sort_bool::true_();
+      }
+      else if (is_tautology() == answer_yes)
+      {
+        mCRL2log(log::debug) << "The formula is a tautology." << std::endl;
+        return sort_bool::false_();
       }
       else
       {
-        mCRL2log(debug) << "The formula is a contradiction or a tautology." << std::endl;
-        v_result = 0;
+        mCRL2log(log::debug) << "The formula is satisfiable, but not a tautology." << std::endl;
+        data_expression t=get_branch(f_bdd, true);
+        if (t==data_expression())
+        { throw mcrl2::runtime_error(
+            "Cannot provide witness. This is probably caused by an abrupt stop of the\n"
+            "conversion from expression to EQ-BDD. This typically occurs when a time limit is set.");
+        }
+        return t;
       }
-      return v_result;
     }
 
     /// \brief Returns all the guards on a path in the BDD that leads to a leaf labelled "false", if such a leaf exists.
-    virtual ATermAppl get_counter_example()
+    virtual data_expression get_counter_example()
     {
-      ATermAppl v_result;
-
       update_answers();
-      if (!(is_contradiction() == answer_yes) && !(is_tautology() == answer_yes))
+      if (is_contradiction() == answer_yes)
       {
-        mCRL2log(debug) << "The formula appears to be satisfiable." << std::endl;
-        v_result = get_branch(f_bdd, false);
+        mCRL2log(log::debug) << "The formula is a contradiction." << std::endl;
+        return sort_bool::false_();
+      }
+      else if (is_tautology() == answer_yes)
+      {
+        mCRL2log(log::debug) << "The formula is a tautology." << std::endl;
+        return sort_bool::true_();
       }
       else
       {
-        mCRL2log(debug) << "The formula is a contradiction or a tautology." << std::endl;
-        v_result = 0;
+        mCRL2log(log::debug) << "The formula is satisfiable, but not a tautology." << std::endl;
+        data_expression t=get_branch(f_bdd, false);
+        if (t==data_expression())
+        { throw mcrl2::runtime_error(
+            "Cannot provide counter example. This is probably caused by an abrupt stop of the\n"
+            "conversion from expression to EQ-BDD. This typically occurs when a time limit is set.");
+        }
+        return t;
       }
-      if (v_result==0)
-      {
-        throw mcrl2::runtime_error(
-          "Cannot provide counter example. This is probably caused by an abrupt stop of the\n"
-          "conversion from expression to EQ-BDD. This typically occurs when a time limit is set.");
-      }
-      return v_result;
     }
 
 };
