@@ -71,7 +71,7 @@ void test_representative_generator()
 
   // Check whether the representative of the set of reals is the empty set of reals.
   BOOST_CHECK(default_expression_generator(container_sort(set_container(),data::sort_real::real_())) ==
-              data::sort_set::emptyset(data::sort_real::real_()));
+              data::sort_set::empty(data::sort_real::real_()));
 
 
 }
