@@ -18,13 +18,11 @@
 #include "mcrl2/utilities/logger.h"
 #include "mcrl2/atermpp/convert.h"
 #include "mcrl2/data/standard_utility.h"
-#include "mcrl2/data/postfix_identifier_generator.h"
-#include "mcrl2/data/fresh_variable_generator.h"
+#include "mcrl2/data/set_identifier_generator.h"
 #include "mcrl2/data/replace.h"
 #include "mcrl2/data/classic_enumerator.h"
 #include "mcrl2/lps/detail/lps_algorithm.h"
 #include "mcrl2/lps/replace.h"
-#include "mcrl2/lps/find.h"
 
 namespace mcrl2
 {
@@ -110,9 +108,9 @@ class binary_algorithm: public lps::detail::lps_algorithm
       data::variable_list process_parameters = m_spec.process().process_parameters();
       data::variable_vector new_parameters;
 
-      mCRL2log(debug) << "Original process parameters: " << data::pp(process_parameters) << std::endl;
+      mCRL2log(log::debug) << "Original process parameters: " << data::pp(process_parameters) << std::endl;
 
-      data::fresh_variable_generator<> generator;
+      data::set_identifier_generator generator;
       generator.add_identifiers(lps::find_identifiers(m_spec));
       enumerator_type enumerator(m_spec.data(),m_rewriter);
 
@@ -127,7 +125,9 @@ class binary_algorithm: public lps::detail::lps_algorithm
           data::data_expression_vector enumerated_elements; // List to store enumerated elements of a parameter
 
           // for (enumerator_type j(enumerator_type(m_spec.data(),par,m_rewriter,data::data_expression(data::sort_bool::true_()))); j != enumerator_type() ; ++j)
-          for (enumerator_type::iterator j=enumerator.begin(push_front(data::variable_list(),par),data::data_expression(data::sort_bool::true_())); 
+          for (enumerator_type::iterator j=enumerator.begin(
+                           push_front(data::variable_list(),par),
+                           data::data_expression(data::sort_bool::true_()));
                 j != enumerator.end() ; ++j)
           {
             enumerated_elements.push_back((*j)(par));
@@ -136,23 +136,23 @@ class binary_algorithm: public lps::detail::lps_algorithm
           m_enumerated_elements[par] = enumerated_elements;
 
           //Calculate the number of booleans needed to encode par
-          int n = static_cast< int >(ceil(log(static_cast< double >(enumerated_elements.size())) / log(static_cast< double >(2))));
+          int n = static_cast< int >(ceil(::log(static_cast< double >(enumerated_elements.size())) / ::log(static_cast< double >(2))));
 
           //Set hint for fresh variable names
-          generator.set_hint(par.name());
+          std::string par_name = par.name();
 
           // Temp list for storage
           data::variable_vector new_pars;
           //Create new parameters and add them to the parameter list.
           for (int i = 0; i<n; ++i)
           {
-            data::variable v(generator(data::sort_bool::bool_()));
+            data::variable v(generator(par_name), data::sort_bool::bool_());
             new_parameters.push_back(v);
             new_pars.push_back(v);
           }
           // n = new_pars.size() && new_pars.size() = ceil(log_2(j)) && new_pars.size() = ceil(log_2(enumerated_elements.size()))
 
-          mCRL2log(verbose) << "Parameter " << data::pp(par) << ":" << data::pp(par.sort()) << " has been replaced by " << new_pars.size() << " parameter(s) " << data::pp(new_pars) << " of sort Bool" << std::endl;
+          mCRL2log(log::verbose) << "Parameter " << data::pp(par) << ":" << data::pp(par.sort()) << " has been replaced by " << new_pars.size() << " parameter(s) " << data::pp(new_pars) << " of sort Bool" << std::endl;
 
           //Store new parameters in a hastable
           m_new_parameters[par]=new_pars;
@@ -166,7 +166,7 @@ class binary_algorithm: public lps::detail::lps_algorithm
         }
       }
 
-      mCRL2log(debug) << "New process parameter(s): " << data::pp(new_parameters) << std::endl;
+      mCRL2log(log::debug) << "New process parameter(s): " << data::pp(new_parameters) << std::endl;
 
       m_spec.process().process_parameters() = atermpp::convert<data::variable_list>(new_parameters);
     }
@@ -192,7 +192,7 @@ class binary_algorithm: public lps::detail::lps_algorithm
           data::variable_vector new_parameters = m_new_parameters[i->lhs()];
           data::data_expression_vector elements = m_enumerated_elements[i->lhs()];
 
-          mCRL2log(debug) << "Found " << new_parameters.size() << " new parameter(s) for parameter " << data::pp(i->lhs()) << std::endl;
+          mCRL2log(log::debug) << "Found " << new_parameters.size() << " new parameter(s) for parameter " << data::pp(i->lhs()) << std::endl;
 
           for (size_t j = 0; j < new_parameters.size(); ++j)
           {
@@ -227,7 +227,7 @@ class binary_algorithm: public lps::detail::lps_algorithm
         }
       }
 
-      mCRL2log(debug) << "Replaced assignment(s) " << data::pp(v) << " with assignment(s) " << data::pp(result) << std::endl;
+      mCRL2log(log::debug) << "Replaced assignment(s) " << data::pp(v) << " with assignment(s) " << data::pp(result) << std::endl;
 
       return atermpp::convert<data::assignment_list>(result);
     }
@@ -271,12 +271,12 @@ class binary_algorithm: public lps::detail::lps_algorithm
       replace_enumerated_parameters();
 
       // Initial process
-      mCRL2log(debug) << "Updating process initializer" << std::endl;
+      mCRL2log(log::debug) << "Updating process initializer" << std::endl;
 
       m_spec.initial_process() = process_initializer(replace_enumerated_parameters_in_assignments(m_spec.initial_process().assignments()));
 
       // Summands
-      mCRL2log(debug) << "Updating summands" << std::endl;
+      mCRL2log(log::debug) << "Updating summands" << std::endl;
 
       std::for_each(m_spec.process().action_summands().begin(),
                     m_spec.process().action_summands().end(),
