@@ -136,42 +136,6 @@ class multi_action
       return front(m_actions).arguments();
     }
 
-    /// \brief Returns a string representation of the multi action
-    std::string to_string() const
-    {
-      std::string result;
-      if (m_actions.size()==0)
-      {
-        result="tau";
-      }
-      else
-      {
-        if (has_time() && m_actions.size()>1)
-        {
-          result="(";
-        }
-        for (action_list::const_iterator i=m_actions.begin(); i!=m_actions.end(); ++i)
-        {
-          result = result+core::pp(*i);
-          action_list::const_iterator i_next=i;
-          i_next++;
-          if (i_next!=m_actions.end())
-          {
-            result=result+"|";
-          }
-        }
-      }
-      if (has_time())
-      {
-        if (m_actions.size()>1)
-        {
-          result=result+")";
-        }
-        result=result+("@ " + core::pp(m_time));
-      }
-      return result;
-    }
-
     /// \brief Joins the actions of both multi actions.
     /// \pre The time of both multi actions must be equal.
     multi_action operator+(const multi_action& other) const
@@ -206,6 +170,13 @@ bool is_multi_action(const atermpp::aterm_appl& t)
   return core::detail::gsIsMultAct(t);
 }
 
+// template function overloads
+std::string pp(const multi_action& x);
+void normalize_sorts(multi_action& x, const data::data_specification& dataspec);
+void translate_user_notation(lps::multi_action& x);
+std::set<data::variable> find_variables(const lps::multi_action& x);
+std::set<data::variable> find_free_variables(const lps::multi_action& x);
+
 /// \cond INTERNAL_DOCS
 namespace detail
 {
@@ -217,7 +188,7 @@ atermpp::aterm_appl multi_action_to_aterm(const multi_action& m)
 {
   return core::detail::gsMakeMultAct(m.actions());
 }
-   
+
 /// \brief Visits all permutations of the arrays, and calls f for each instance.
 /// \pre The range [first, last) contains sorted arrays.
 /// \param first Start of a sequence of arrays
@@ -330,7 +301,7 @@ struct equal_data_parameters_builder
     }
     data::data_expression expr = d::lazy::join_and(v.begin(), v.end());
 #ifdef MCRL2_EQUAL_MULTI_ACTIONS_DEBUG
-    std::cerr << "  <and-term> " << data::pp(expr) << std::endl;
+    mCRL2log(debug) << "  <and-term> " << data::pp(expr) << std::endl;
 #endif
     result.insert(expr);
   }
@@ -377,13 +348,6 @@ struct not_equal_multi_actions_builder
 } // namespace detail
 /// \endcond
 
-/// \brief Returns a string representation of a multi action
-inline
-std::string pp1(const multi_action& m)
-{
-  return m.to_string();
-}
-
 /// \brief Returns a data expression that expresses under which conditions the
 /// multi actions a and b are equal. The multi actions may contain free variables.
 /// \param a A sequence of actions
@@ -392,9 +356,9 @@ std::string pp1(const multi_action& m)
 inline data::data_expression equal_multi_actions(const multi_action& a, const multi_action& b)
 {
 #ifdef MCRL2_EQUAL_MULTI_ACTIONS_DEBUG
-  std::cerr << "\n<equal multi actions>" << std::endl;
-  std::cerr << "a = " << lps::pp(a.actions()) << std::endl;
-  std::cerr << "b = " << lps::pp(b.actions()) << std::endl;
+  mCRL2log(debug) << "\n<equal multi actions>" << std::endl;
+  mCRL2log(debug) << "a = " << lps::pp(a.actions()) << std::endl;
+  mCRL2log(debug) << "b = " << lps::pp(b.actions()) << std::endl;
 #endif
   using namespace data::lazy;
 
@@ -407,9 +371,9 @@ inline data::data_expression equal_multi_actions(const multi_action& a, const mu
   if (!detail::equal_action_signatures(va, vb))
   {
 #ifdef MCRL2_EQUAL_MULTI_ACTIONS_DEBUG
-    std::cerr << "different action signatures detected!" << std::endl;
-    std::cerr << "a = " << action_list(va.begin(), va.end()) << std::endl;
-    std::cerr << "b = " << action_list(vb.begin(), vb.end()) << std::endl;
+    mCRL2log(debug) << "different action signatures detected!" << std::endl;
+    mCRL2log(debug) << "a = " << action_list(va.begin(), va.end()) << std::endl;
+    mCRL2log(debug) << "b = " << action_list(vb.begin(), vb.end()) << std::endl;
 #endif
     return data::sort_bool::false_();
   }
