@@ -36,9 +36,6 @@ namespace mcrl2
 namespace lps
 {
 
-template <typename Container, typename OutputIterator>
-void find_sort_expressions(Container const& container, OutputIterator o);
-
 template <typename Object>
 bool is_well_typed(const Object& o);
 
@@ -172,7 +169,6 @@ class specification
       // LPSs it takes too much time
       assert(is_well_typed(*this));
       specification tmp(*this);
-      // tmp.data() = data::remove_all_system_defined(tmp.data());
       core::detail::save_aterm(specification_to_aterm(tmp), filename, binary);
     }
 
@@ -254,13 +250,21 @@ class specification
     }
 };
 
+// template function overloads
+std::string pp(const specification& x);
+std::string pp_with_summand_numbers(const specification& x);
+std::set<data::sort_expression> find_sort_expressions(const lps::specification& x);
+std::set<data::variable> find_variables(const lps::specification& x);
+std::set<data::variable> find_free_variables(const lps::specification& x);
+std::set<data::function_symbol> find_function_symbols(const lps::specification& x);
+std::set<core::identifier_string> find_identifiers(const lps::specification& x);
+
 /// \brief Adds all sorts that appear in the process of l to the data specification of l.
 /// \param l A linear process specification
 inline
 void complete_data_specification(lps::specification& spec)
 {
-  std::set<data::sort_expression> s;
-  lps::find_sort_expressions(spec, std::inserter(s, s.end()));
+  std::set<data::sort_expression> s = lps::find_sort_expressions(spec);
   spec.data().add_context_sorts(s);
 }
 
@@ -296,12 +300,8 @@ bool operator!=(const specification& spec1, const specification& spec2)
 
 } // namespace mcrl2
 
-#ifndef MCRL2_LPS_FIND_H
-#include "mcrl2/lps/find.h"
-#endif
-
 #ifndef MCRL2_LPS_DETAIL_LPS_WELL_TYPED_CHECKER_H
 #include "mcrl2/lps/detail/lps_well_typed_checker.h"
 #endif
 
-#endif // MCRL2_LPS_SPECIFICATION_H                                                                                       
+#endif // MCRL2_LPS_SPECIFICATION_H
