@@ -44,7 +44,7 @@ void run_linearisation_instance(const std::string& spec, const t_lin_options& op
 void run_linearisation_test_case(const std::string& spec, const bool expect_success = true)
 {
   // Set various rewrite strategies
-  rewrite_strategy_vector rewrite_strategies = utilities::get_test_rewrite_strategies();
+  rewrite_strategy_vector rewrite_strategies = utilities::get_test_rewrite_strategies(false);
 
   for (rewrite_strategy_vector::const_iterator i = rewrite_strategies.begin(); i != rewrite_strategies.end(); ++i)
   {
@@ -1073,6 +1073,36 @@ BOOST_AUTO_TEST_CASE(test_bug_775b)
   run_linearisation_test_case(spec);
 }
 
+BOOST_AUTO_TEST_CASE(test_bug_alphabet_reduction)
+{
+  const std::string spec =
+    "sort Variables = struct s;\n"
+    "\n"
+    "act send_c, recv_c, comm_c: Bool;\n"
+    "    chng: Set(Variables);\n"
+    "\n"
+    "init allow( { comm_c | chng | chng } ,\n"
+    "  comm( { send_c | recv_c -> comm_c,\n"
+    "          chng | chng -> chng},\n"
+    "          send_c(false) | chng( {} ) || recv_c(false) | chng( {s} )\n"
+    "       ) );\n";
+  run_linearisation_test_case(spec);
+}
+
+BOOST_AUTO_TEST_CASE(test_bug_alphabet_reduction1)
+{
+  const std::string spec =
+    "act a: Bool;\n"
+    "    b: Nat;\n"
+    "\n"
+    "proc X = allow( {b | a | a },\n"
+    "          a(true)| b(5) | a(false) );\n"
+    "\n"
+    "init X;\n";
+  run_linearisation_test_case(spec);
+}
+
+
 boost::unit_test::test_suite* init_unit_test_suite(int argc, char* argv[])
 {
   MCRL2_ATERMPP_INIT(argc, argv)
