@@ -32,6 +32,7 @@
 
 using namespace mcrl2::core;
 using namespace mcrl2::core::detail;
+using namespace mcrl2::log;
 
 #ifdef USE_BCG
 // #include <bcg_user.h>
@@ -51,13 +52,6 @@ namespace detail
 
 atermpp::vector < ATermAppl > state_label_lts::vector_templates;
 
-lps::specification const& empty_specification()
-{
-  static lps::specification dummy;
-
-  return dummy;
-}
-
 lts_type guess_format(string const& s)
 {
   string::size_type pos = s.find_last_of('.');
@@ -79,7 +73,9 @@ lts_type guess_format(string const& s)
     else if (ext == "svc")
     {
       mCRL2log(verbose) << "Detected SVC extension; assuming mCRL2 format.\n";
-      return lts_lts;
+      return lts_lts; /* This is wrong, and must be changed to lts_svc 
+                         However, as a consequence the translations from and to 
+                         .svc format must also be implemented. */
     }
     else if (ext == "fsm")
     {
