@@ -20,14 +20,12 @@
 #include "mcrl2/utilities/reachable_nodes.h"
 #include "mcrl2/utilities/logger.h"
 #include "mcrl2/utilities/detail/iota.h"
-#include "mcrl2/data/find.h"
 #include "mcrl2/data/utility.h"
 #include "mcrl2/data/detail/assignment_functional.h"
 #include "mcrl2/data/detail/sorted_sequence_algorithm.h"
 #include "mcrl2/pbes/detail/free_variable_visitor.h"
 #include "mcrl2/pbes/pbes.h"
 #include "mcrl2/pbes/remove_parameters.h"
-#include "mcrl2/pbes/find.h"
 
 namespace mcrl2
 {
@@ -80,29 +78,6 @@ class pbes_parelm_algorithm
 
     /// \cond INTERNAL_DOCS
 
-    /// \brief Prints a container of terms to standard error
-    /// \param v A container
-    /// \param message A string
-    /// \param print_index If true, an index is written in front of each term
-    template <typename Container>
-    void print_pp_container(const Container& v, std::string message = "<variables>", bool print_index = false) const
-    {
-      std::cerr << message << std::endl;
-      int index = 0;
-      for (typename Container::const_iterator i = v.begin(); i != v.end(); ++i)
-      {
-        if (print_index)
-        {
-          std::cerr << index++ << " " << mcrl2::core::pp(*i) << std::endl;
-        }
-        else
-        {
-          std::cerr << mcrl2::core::pp(*i) << " ";
-        }
-      }
-      std::cerr << std::endl;
-    }
-
     /// \brief FUNCTION_DESCRIPTION
     // \brief Prints a container to standard error
     // \param v A container
@@ -190,7 +165,7 @@ class pbes_parelm_algorithm
           int k = variable_index(i->variable().parameters(), *j);
           if (k < 0)
           {
-            std::cerr << "<variable error>" << mcrl2::core::pp(*j) << std::endl;
+            std::cerr << "<variable error>" << data::pp(*j) << std::endl;
             continue;
           }
           v.insert(offset + k);
@@ -259,16 +234,16 @@ class pbes_parelm_algorithm
       }
 
       // print debug output
-      if (mCRL2logEnabled(debug))
+      if (mCRL2logEnabled(log::debug))
       {
-        mCRL2log(debug) << "\ninfluential parameters:" << std::endl;
+        mCRL2log(log::debug) << "\ninfluential parameters:" << std::endl;
         for (std::set<size_t>::iterator i = v.begin(); i != v.end(); ++i)
         {
           core::identifier_string X1 = find_predicate_variable(p, *i);
           data::variable v1 = predicate_variables[*i];
-          mCRL2log(debug) << "(" + mcrl2::core::pp(X1) + ", " + mcrl2::core::pp(v1) + ")\n";
+          mCRL2log(log::debug) << "(" + core::pp(X1) + ", " + data::pp(v1) + ")\n";
         }
-        mCRL2log(debug) << "\ndependencies:" << std::endl;
+        mCRL2log(log::debug) << "\ndependencies:" << std::endl;
         typedef typename boost::graph_traits<graph>::edge_iterator edge_iterator;
         std::pair<edge_iterator, edge_iterator> e = edges(G);
         edge_iterator first = e.first;
@@ -282,16 +257,16 @@ class pbes_parelm_algorithm
           size_t i2 = boost::target(e, G);
           core::identifier_string X2 = find_predicate_variable(p, i2);
           data::variable v2 = predicate_variables[i2];
-          std::string left  = "(" + mcrl2::core::pp(X1) + ", " + mcrl2::core::pp(v1) + ")";
-          std::string right = "(" + mcrl2::core::pp(X2) + ", " + mcrl2::core::pp(v2) + ")";
-          mCRL2log(debug) << left << " -> " << right << std::endl;
+          std::string left  = "(" + core::pp(X1) + ", " + data::pp(v1) + ")";
+          std::string right = "(" + core::pp(X2) + ", " + data::pp(v2) + ")";
+          mCRL2log(log::debug) << left << " -> " << right << std::endl;
         }
       }
 
       // print verbose output
-      if (mCRL2logEnabled(verbose))
+      if (mCRL2logEnabled(log::verbose))
       {
-        mCRL2log(verbose) << "\nremoving the following parameters:" << std::endl;
+        mCRL2log(log::verbose) << "\nremoving the following parameters:" << std::endl;
         for (std::map<core::identifier_string, std::vector<size_t> >::const_iterator i = removals.begin(); i != removals.end(); ++i)
         {
           core::identifier_string X1 = i->first;
@@ -299,7 +274,7 @@ class pbes_parelm_algorithm
           for (std::vector<size_t>::const_iterator j = (i->second).begin(); j != (i->second).end(); ++j)
           {
             data::variable v1 = predicate_variables[*j + propvar_offsets[X1]];
-            mCRL2log(verbose) << "(" + mcrl2::core::pp(X1) + ", " + mcrl2::core::pp(v1) + ")\n";
+            mCRL2log(log::verbose) << "(" + core::pp(X1) + ", " + data::pp(v1) + ")\n";
           }
         }
       }
