@@ -22,7 +22,7 @@
 #include "mcrl2/core/detail/struct_core.h"
 #include "mcrl2/core/detail/constructors.h"
 #include "mcrl2/core/detail/soundness_checks.h"
-#include "mcrl2/data/print.h"
+#include "mcrl2/data/data_specification.h"
 #include "mcrl2/pbes/propositional_variable.h"
 #include "mcrl2/pbes/detail/free_variable_visitor.h"
 #include "mcrl2/pbes/detail/compare_pbes_expression_visitor.h"
@@ -352,10 +352,26 @@ class exists: public pbes_expression
 };
 //--- end generated classes ---//
 
+// template function overloads
+std::string pp(const core::identifier_string& x);
+std::string pp(const pbes_expression& x);
+std::string pp(const pbes_expression_list& x);
+std::string pp(const pbes_expression_vector& x);
+std::string pp(const propositional_variable_instantiation& x);
+std::string pp(const propositional_variable_instantiation_list& x);
+std::string pp(const propositional_variable_instantiation_vector& x);
+std::set<pbes_system::propositional_variable_instantiation> find_propositional_variable_instantiations(const pbes_system::pbes_expression& x);
+std::set<core::identifier_string> find_identifiers(const pbes_system::pbes_expression& x);
+bool search_variable(const pbes_system::pbes_expression& x, const data::variable& v);
+
+// TODO: These should be removed when the ATerm code has been replaced.
+std::string pp(const atermpp::aterm& x);
+std::string pp(const atermpp::aterm_appl& x);
+
 /// \brief Returns true if the term t is equal to true
 /// \param t A PBES expression
 /// \return True if the term t is equal to true
-inline bool is_pbes_true(pbes_expression t)
+inline bool is_pbes_true(const pbes_expression& t)
 {
   return core::detail::gsIsPBESTrue(t);
 }
@@ -363,7 +379,7 @@ inline bool is_pbes_true(pbes_expression t)
 /// \brief Returns true if the term t is equal to false
 /// \param t A PBES expression
 /// \return True if the term t is equal to false
-inline bool is_pbes_false(pbes_expression t)
+inline bool is_pbes_false(const pbes_expression& t)
 {
   return core::detail::gsIsPBESFalse(t);
 }
@@ -371,7 +387,7 @@ inline bool is_pbes_false(pbes_expression t)
 /// \brief Returns true if the term t is a not expression
 /// \param t A PBES expression
 /// \return True if the term t is a not expression
-inline bool is_pbes_not(pbes_expression t)
+inline bool is_pbes_not(const pbes_expression& t)
 {
   return core::detail::gsIsPBESNot(t);
 }
@@ -380,7 +396,7 @@ inline bool is_pbes_not(pbes_expression t)
 /// \brief Returns true if the term t is an and expression
 /// \param t A PBES expression
 /// \return True if the term t is an and expression
-inline bool is_pbes_and(pbes_expression t)
+inline bool is_pbes_and(const pbes_expression& t)
 {
   return core::detail::gsIsPBESAnd(t);
 }
@@ -388,7 +404,7 @@ inline bool is_pbes_and(pbes_expression t)
 /// \brief Returns true if the term t is an or expression
 /// \param t A PBES expression
 /// \return True if the term t is an or expression
-inline bool is_pbes_or(pbes_expression t)
+inline bool is_pbes_or(const pbes_expression& t)
 {
   return core::detail::gsIsPBESOr(t);
 }
@@ -396,7 +412,7 @@ inline bool is_pbes_or(pbes_expression t)
 /// \brief Returns true if the term t is an imp expression
 /// \param t A PBES expression
 /// \return True if the term t is an imp expression
-inline bool is_pbes_imp(pbes_expression t)
+inline bool is_pbes_imp(const pbes_expression& t)
 {
   return core::detail::gsIsPBESImp(t);
 }
@@ -404,7 +420,7 @@ inline bool is_pbes_imp(pbes_expression t)
 /// \brief Returns true if the term t is a universal quantification
 /// \param t A PBES expression
 /// \return True if the term t is a universal quantification
-inline bool is_pbes_forall(pbes_expression t)
+inline bool is_pbes_forall(const pbes_expression& t)
 {
   return core::detail::gsIsPBESForall(t);
 }
@@ -412,7 +428,7 @@ inline bool is_pbes_forall(pbes_expression t)
 /// \brief Returns true if the term t is an existential quantification
 /// \param t A PBES expression
 /// \return True if the term t is an existential quantification
-inline bool is_pbes_exists(pbes_expression t)
+inline bool is_pbes_exists(const pbes_expression& t)
 {
   return core::detail::gsIsPBESExists(t);
 }
@@ -436,7 +452,7 @@ inline bool is_false(const pbes_expression& t)
 /// \brief Test for a negation
 /// \param t A PBES expression
 /// \return True if it is a negation
-inline bool is_not(pbes_expression t)
+inline bool is_not(const pbes_expression& t)
 {
   return is_pbes_not(t);
 }
@@ -444,23 +460,40 @@ inline bool is_not(pbes_expression t)
 /// \brief Test for a conjunction
 /// \param t A PBES expression
 /// \return True if it is a conjunction
-inline bool is_and(pbes_expression t)
+inline bool is_and(const pbes_expression& t)
 {
   return is_pbes_and(t);
 }
 
+
+/// \brief Test for a conjunction
+/// \param t A PBES expression or a data expression
+/// \return True if it is a conjunction
+inline bool data_is_and(const pbes_expression& t)
+{
+  return is_pbes_and(t) || data::sort_bool::is_and_application(t);
+}
+
 /// \brief Test for a disjunction
 /// \param t A PBES expression
 /// \return True if it is a disjunction
-inline bool is_or(pbes_expression t)
+inline bool is_or(const pbes_expression& t)
 {
   return is_pbes_or(t);
 }
 
+/// \brief Test for a disjunction
+/// \param t A PBES expression or a data expression
+/// \return True if it is a disjunction
+inline bool data_is_or(const pbes_expression& t)
+{
+  return is_pbes_or(t) || data::sort_bool::is_or_application(t);
+}
+
 /// \brief Test for an implication
 /// \param t A PBES expression
 /// \return True if it is an implication
-inline bool is_imp(pbes_expression t)
+inline bool is_imp(const pbes_expression& t)
 {
   return is_pbes_imp(t);
 }
@@ -468,7 +501,7 @@ inline bool is_imp(pbes_expression t)
 /// \brief Test for an universal quantification
 /// \param t A PBES expression
 /// \return True if it is a universal quantification
-inline bool is_forall(pbes_expression t)
+inline bool is_forall(const pbes_expression& t)
 {
   return is_pbes_forall(t);
 }
@@ -476,7 +509,7 @@ inline bool is_forall(pbes_expression t)
 /// \brief Test for an existential quantification
 /// \param t A PBES expression
 /// \return True if it is an existential quantification
-inline bool is_exists(pbes_expression t)
+inline bool is_exists(const pbes_expression& t)
 {
   return is_pbes_exists(t);
 }
@@ -508,18 +541,22 @@ int precedence(const pbes_expression& x)
   {
     return 0;
   }
-  else if (is_imp(x)) 
-  {
-    return 1;
-  }
-  else if (is_and(x) || is_or(x)) 
+  else if (is_imp(x))
   {
     return 2;
   }
-  else if (is_not(x)) 
+  else if (is_or(x))
   {
     return 3;
   }
+  else if (is_and(x))
+  {
+    return 4;
+  }
+  else if (is_not(x))
+  {
+    return 5;
+  }
   return core::detail::precedences::max_precedence;
 }
 
@@ -530,6 +567,8 @@ inline int precedence(const imp& x)      { return precedence(static_cast<const p
 inline int precedence(const and_& x)     { return precedence(static_cast<const pbes_expression&>(x)); }
 inline int precedence(const or_& x)      { return precedence(static_cast<const pbes_expression&>(x)); }
 inline int precedence(const not_& x)     { return precedence(static_cast<const pbes_expression&>(x)); }
+inline int precedence(const data::data_expression & x)               { return precedence(static_cast<const pbes_expression&>(x)); }
+inline int precedence(const propositional_variable_instantiation& x) { return precedence(static_cast<const pbes_expression&>(x)); }
 
 /// \brief Returns true if the operations have the same precedence, but are different
 template <typename T1, typename T2>
@@ -561,7 +600,7 @@ namespace accessors
 /// \param t A PBES expression
 /// \return The converted expression
 inline
-data::data_expression val(pbes_expression t)
+data::data_expression val(const pbes_expression& t)
 {
   assert(data::is_data_expression(t));
   return atermpp::aterm_appl(t);
@@ -571,7 +610,7 @@ data::data_expression val(pbes_expression t)
 /// \param t A PBES expression
 /// \return The pbes expression argument of expressions of type not, exists and forall.
 inline
-pbes_expression arg(pbes_expression t)
+pbes_expression arg(const pbes_expression& t)
 {
   if (is_pbes_not(t))
   {
@@ -584,30 +623,78 @@ pbes_expression arg(pbes_expression t)
   }
 }
 
+/// \brief Returns the pbes expression argument of expressions of type not, exists and forall.
+/// \param t A PBES expression or a data expression
+/// \return The pbes expression argument of expressions of type not, exists and forall.
+inline
+pbes_expression data_arg(const pbes_expression& t)
+{
+  if (data::is_data_expression(t))
+  {
+    return data::application(t).arguments().front();
+  }
+  else
+  {
+    return arg(t);
+  }
+}
+
 /// \brief Returns the left hand side of an expression of type and, or or imp.
 /// \param t A PBES expression
 /// \return The left hand side of an expression of type and, or or imp.
 inline
-pbes_expression left(pbes_expression t)
+pbes_expression left(const pbes_expression& t)
 {
   assert(is_and(t) || is_or(t) || is_imp(t));
   return atermpp::arg1(t);
 }
 
+/// \brief Returns the left hand side of an expression of type and, or or imp.
+/// \param t A PBES expression or a data expression
+/// \return The left hand side of an expression of type and, or or imp.
+inline
+pbes_expression data_left(const pbes_expression& t)
+{
+  if (data::is_data_expression(t))
+  {
+    return data::application(t).left();
+  }
+  else
+  {
+    return left(t);
+  }
+}
+
 /// \brief Returns the right hand side of an expression of type and, or or imp.
 /// \param t A PBES expression
 /// \return The right hand side of an expression of type and, or or imp.
 inline
-pbes_expression right(pbes_expression t)
+pbes_expression right(const pbes_expression& t)
 {
   return atermpp::arg2(t);
 }
 
+/// \brief Returns the left hand side of an expression of type and, or or imp.
+/// \param t A PBES expression or a data expression
+/// \return The left hand side of an expression of type and, or or imp.
+inline
+pbes_expression data_right(const pbes_expression& t)
+{
+  if (data::is_data_expression(t))
+  {
+    return data::application(t).right();
+  }
+  else
+  {
+    return right(t);
+  }
+}
+
 /// \brief Returns the variables of a quantification expression
 /// \param t A PBES expression
 /// \return The variables of a quantification expression
 inline
-data::variable_list var(pbes_expression t)
+data::variable_list var(const pbes_expression& t)
 {
   assert(is_forall(t) || is_exists(t));
   return data::variable_list(atermpp::term_list_iterator< data::variable >(atermpp::list_arg1(t)),
@@ -618,7 +705,7 @@ data::variable_list var(pbes_expression t)
 /// \param t A PBES expression
 /// \return The name of a propositional variable expression
 inline
-core::identifier_string name(pbes_expression t)
+core::identifier_string name(const pbes_expression& t)
 {
   assert(is_propositional_variable_instantiation(t));
   return atermpp::arg1(t);
@@ -628,7 +715,7 @@ core::identifier_string name(pbes_expression t)
 /// \param t A PBES expression
 /// \return The parameters of a propositional variable instantiation.
 inline
-data::data_expression_list param(pbes_expression t)
+data::data_expression_list param(const pbes_expression& t)
 {
   assert(is_propositional_variable_instantiation(t));
   return data::data_expression_list(atermpp::term_list_iterator< data::data_expression >(atermpp::list_arg2(t)),
@@ -660,7 +747,7 @@ pbes_expression false_()
 /// \param p A PBES expression
 /// \return The value <tt>!p</tt>
 inline
-pbes_expression not_(pbes_expression p)
+pbes_expression not_(const pbes_expression& p)
 {
   return pbes_expression(core::detail::gsMakePBESNot(p));
 }
@@ -670,7 +757,7 @@ pbes_expression not_(pbes_expression p)
 /// \param q A PBES expression
 /// \return The value <tt>p && q</tt>
 inline
-pbes_expression and_(pbes_expression p, pbes_expression q)
+pbes_expression and_(const pbes_expression& p, const pbes_expression& q)
 {
   return pbes_expression(core::detail::gsMakePBESAnd(p,q));
 }
@@ -680,7 +767,7 @@ pbes_expression and_(pbes_expression p, pbes_expression q)
 /// \param q A PBES expression
 /// \return The value <tt>p || q</tt>
 inline
-pbes_expression or_(pbes_expression p, pbes_expression q)
+pbes_expression or_(const pbes_expression& p, const pbes_expression& q)
 {
   return pbes_expression(core::detail::gsMakePBESOr(p,q));
 }
@@ -690,7 +777,7 @@ pbes_expression or_(pbes_expression p, pbes_expression q)
 /// \param q A PBES expression
 /// \return The value <tt>p => q</tt>
 inline
-pbes_expression imp(pbes_expression p, pbes_expression q)
+pbes_expression imp(const pbes_expression& p, const pbes_expression& q)
 {
   return pbes_expression(core::detail::gsMakePBESImp(p,q));
 }
@@ -700,7 +787,7 @@ pbes_expression imp(pbes_expression p, pbes_expression q)
 /// \param p A PBES expression
 /// \return The value <tt>forall l.p</tt>
 inline
-pbes_expression forall(data::variable_list l, pbes_expression p)
+pbes_expression forall(const data::variable_list& l, const pbes_expression& p)
 {
   if (l.empty())
   {
@@ -714,7 +801,7 @@ pbes_expression forall(data::variable_list l, pbes_expression p)
 /// \param p A PBES expression
 /// \return The value <tt>exists l.p</tt>
 inline
-pbes_expression exists(data::variable_list l, pbes_expression p)
+pbes_expression exists(const data::variable_list& l, const pbes_expression& p)
 {
   if (l.empty())
   {
@@ -750,11 +837,20 @@ pbes_expression join_and(FwdIt first, FwdIt last)
 /// \param expr A PBES expression
 /// \return A sequence of operands
 inline
-atermpp::set<pbes_expression> split_or(const pbes_expression& expr)
+atermpp::set<pbes_expression> split_or(const pbes_expression& expr, bool split_data_expressions = false)
 {
   using namespace accessors;
   atermpp::set<pbes_expression> result;
-  utilities::detail::split(expr, std::insert_iterator<atermpp::set<pbes_expression> >(result, result.begin()), is_or, left, right);
+
+  if (split_data_expressions)
+  {
+    utilities::detail::split(expr, std::insert_iterator<atermpp::set<pbes_expression> >(result, result.begin()), data_is_or, data_left, data_right);
+  }
+  else
+  {
+    utilities::detail::split(expr, std::insert_iterator<atermpp::set<pbes_expression> >(result, result.begin()), is_or, left, right);
+  }
+
   return result;
 }
 
@@ -765,11 +861,20 @@ atermpp::set<pbes_expression> split_or(const pbes_expression& expr)
 /// \param expr A PBES expression
 /// \return A sequence of operands
 inline
-atermpp::set<pbes_expression> split_and(const pbes_expression& expr)
+atermpp::set<pbes_expression> split_and(const pbes_expression& expr, bool split_data_expressions = false)
 {
   using namespace accessors;
   atermpp::set<pbes_expression> result;
-  utilities::detail::split(expr, std::insert_iterator<atermpp::set<pbes_expression> >(result, result.begin()), is_and, left, right);
+
+  if (split_data_expressions)
+  {
+    utilities::detail::split(expr, std::insert_iterator<atermpp::set<pbes_expression> >(result, result.begin()), data_is_and, data_left, data_right);
+  }
+  else
+  {
+    utilities::detail::split(expr, std::insert_iterator<atermpp::set<pbes_expression> >(result, result.begin()), is_and, left, right);
+  }
+
   return result;
 }
 } // namespace pbes_expr
@@ -785,7 +890,7 @@ using pbes_expr::split_or;
 /// \param p A PBES expression
 /// \return The value <tt>!p</tt>
 inline
-pbes_expression not_(pbes_expression p)
+pbes_expression not_(const pbes_expression& p)
 {
   return utilities::detail::optimized_not(p, pbes_expr::not_, true_(), is_true, false_(), is_false);
 }
@@ -795,7 +900,7 @@ pbes_expression not_(pbes_expression p)
 /// \param q A PBES expression
 /// \return The value <tt>p && q</tt>
 inline
-pbes_expression and_(pbes_expression p, pbes_expression q)
+pbes_expression and_(const pbes_expression& p, const pbes_expression& q)
 {
   return utilities::detail::optimized_and(p, q, pbes_expr::and_, true_(), is_true, false_(), is_false);
 }
@@ -805,7 +910,7 @@ pbes_expression and_(pbes_expression p, pbes_expression q)
 /// \param q A PBES expression
 /// \return The value <tt>p || q</tt>
 inline
-pbes_expression or_(pbes_expression p, pbes_expression q)
+pbes_expression or_(const pbes_expression& p, const pbes_expression& q)
 {
   return utilities::detail::optimized_or(p, q, pbes_expr::or_, true_(), is_true, false_(), is_false);
 }
@@ -815,7 +920,7 @@ pbes_expression or_(pbes_expression p, pbes_expression q)
 /// \param q A PBES expression
 /// \return The value <tt>p => q</tt>
 inline
-pbes_expression imp(pbes_expression p, pbes_expression q)
+pbes_expression imp(const pbes_expression& p, const pbes_expression& q)
 {
   return utilities::detail::optimized_imp(p, q, pbes_expr::imp, not_, true_(), is_true, false_(), is_false);
 }
@@ -846,7 +951,7 @@ inline pbes_expression join_and(FwdIt first, FwdIt last)
 /// \param p A PBES expression
 /// \return The value <tt>forall l.p</tt>
 inline
-pbes_expression forall(data::variable_list l, pbes_expression p)
+pbes_expression forall(const data::variable_list& l, const pbes_expression& p)
 {
   if (l.empty())
   {
@@ -870,7 +975,7 @@ pbes_expression forall(data::variable_list l, pbes_expression p)
 /// \param p A PBES expression
 /// \return The value <tt>exists l.p</tt>
 inline
-pbes_expression exists(data::variable_list l, pbes_expression p)
+pbes_expression exists(const data::variable_list& l, const pbes_expression& p)
 {
   if (l.empty())
   {
@@ -948,13 +1053,13 @@ struct term_traits<pbes_system::pbes_expression>
   /// \param p A term
   /// \return The value <tt>!p</tt>
   static inline
-  term_type not_(term_type p)
+  term_type not_(const term_type& p)
   {
     return atermpp::aterm_appl(core::detail::gsMakePBESNot(p));
   }
 
   static inline
-  bool is_sorted(term_type p, term_type q)
+  bool is_sorted(const term_type& p, const term_type& q)
   {
     pbes_system::detail::compare_pbes_expression_visitor<term_type> pvisitor;
     pvisitor.visit(p);
@@ -980,7 +1085,7 @@ struct term_traits<pbes_system::pbes_expression>
   /// \param q A term
   /// \return The value <tt>p && q</tt>
   static inline
-  term_type and_(term_type p, term_type q)
+  term_type and_(const term_type& p, const term_type& q)
   {
     return atermpp::aterm_appl(core::detail::gsMakePBESAnd(p,q));
   }
@@ -990,7 +1095,7 @@ struct term_traits<pbes_system::pbes_expression>
   /// \param q A term
   /// \return The value <tt>p || q</tt>
   static inline
-  term_type or_(term_type p, term_type q)
+  term_type or_(const term_type& p, const term_type& q)
   {
     return atermpp::aterm_appl(core::detail::gsMakePBESOr(p,q));
   }
@@ -1000,7 +1105,7 @@ struct term_traits<pbes_system::pbes_expression>
   /// \param q A term
   /// \return The value <tt>p && q</tt>, or <tt>q && p</tt>
   static inline
-  term_type sorted_and(term_type p, term_type q)
+  term_type sorted_and(const term_type& p, const term_type& q)
   {
 #ifdef MCRL2_SMART_ARGUMENT_SORTING
     bool sorted = is_sorted(p, q);
@@ -1015,7 +1120,7 @@ struct term_traits<pbes_system::pbes_expression>
   /// \param q A term
   /// \return The value <tt>p || q</tt>
   static inline
-  term_type sorted_or(term_type p, term_type q)
+  term_type sorted_or(const term_type& p, const term_type& q)
   {
 #ifdef MCRL2_SMART_ARGUMENT_SORTING
     bool sorted = is_sorted(p, q);
@@ -1030,7 +1135,7 @@ struct term_traits<pbes_system::pbes_expression>
   /// \param q A term
   /// \return The value <tt>p => q</tt>
   static inline
-  term_type imp(term_type p, term_type q)
+  term_type imp(const term_type& p, const term_type& q)
   {
     return atermpp::aterm_appl(core::detail::gsMakePBESImp(p,q));
   }
@@ -1040,7 +1145,7 @@ struct term_traits<pbes_system::pbes_expression>
   /// \param p A term
   /// \return The value <tt>forall l.p</tt>
   static inline
-  term_type forall(variable_sequence_type l, term_type p)
+  term_type forall(const variable_sequence_type& l, const term_type& p)
   {
     if (l.empty())
     {
@@ -1054,7 +1159,7 @@ struct term_traits<pbes_system::pbes_expression>
   /// \param p A term
   /// \return The value <tt>exists l.p</tt>
   static inline
-  term_type exists(variable_sequence_type l, term_type p)
+  term_type exists(const variable_sequence_type& l, const term_type& p)
   {
     if (l.empty())
     {
@@ -1259,7 +1364,7 @@ struct term_traits<pbes_system::pbes_expression>
   /// \param t A data term
   /// \return The converted term
   static inline
-  term_type dataterm2term(data_term_type t)
+  term_type dataterm2term(const data_term_type& t)
   {
     return t;
   }
@@ -1277,7 +1382,7 @@ struct term_traits<pbes_system::pbes_expression>
   /// \param t A term
   /// \return True if the term is constant
   static inline
-  bool is_constant(term_type /* t */)
+  bool is_constant(const term_type& /* t */)
   {
     return false;
   }
@@ -1288,7 +1393,7 @@ struct term_traits<pbes_system::pbes_expression>
   static inline
   std::string pp(const term_type& t)
   {
-    return core::pp(t);
+    return pbes_system::pp(t);
   }
 };
 
