@@ -22,7 +22,6 @@
 #include "mcrl2/data/data_expression.h"
 #include "mcrl2/data/classic_enumerator.h"
 #include "mcrl2/pbes/pbes.h"
-#include "mcrl2/pbes/find.h"
 #include "mcrl2/pbes/rewriter.h"
 #include "mcrl2/pbes/replace.h"
 #include "mcrl2/pbes/detail/instantiate_global_variables.h"
@@ -118,20 +117,20 @@ core::identifier_string create_propvar_name(core::identifier_string propvar_name
       {
         //If p is a OpId
         propvar_name_current += "@";
-        propvar_name_current += mcrl2::core::pp(*del_i);
+        propvar_name_current += mcrl2::data::pp(*del_i);
       }
       else if (is_application(*del_i))
       {
         // If p is a data application
         propvar_name_current += "@";
-        propvar_name_current += mcrl2::core::pp(*del_i);
+        propvar_name_current += mcrl2::data::pp(*del_i);
       }
       // else if (is_variable(*del_i))
       // { // If p is a freevar
       // }
       else
       {
-        throw mcrl2::runtime_error(std::string("pbesinst: could not rename the variable ") + core::pp(propositional_variable_instantiation(propvar_name, del)));
+        throw mcrl2::runtime_error(std::string("pbesinst: could not rename the variable ") + pbes_system::pp(propositional_variable_instantiation(propvar_name, del)));
       }
     }
   }
@@ -141,6 +140,7 @@ core::identifier_string create_propvar_name(core::identifier_string propvar_name
 
 /// \brief Create a new propositional variable instantiation with instantiated values and infinite variables
 /// \param propvarinst A propositional variable instantiation
+inline
 propositional_variable_instantiation create_naive_propositional_variable_instantiation(propositional_variable_instantiation propvarinst, atermpp::table* enumerated_sorts)
 {
   data::data_expression_list finite_expression;
@@ -163,7 +163,7 @@ propositional_variable_instantiation create_naive_propositional_variable_instant
           "The propositional variable contains a variable of finite sort.\n"
           "Can not handle variables of finite sort when creating a propositional variable name.\n"
           "Computation aborted.\n"
-          "Problematic Term: " + mcrl2::core::pp(*p));
+          "Problematic Term: " + mcrl2::data::pp(*p));
       }
     }
     else
@@ -184,7 +184,7 @@ pbes<> do_lazy_algorithm(pbes<> pbes_spec, PbesRewriter& rewrite)
 {
   // Instantiate free variables in the system
   pbes_system::detail::instantiate_global_variables(pbes_spec);
-  mCRL2log(verbose) << "Using lazy approach..." << std::endl;
+  mCRL2log(log::verbose) << "Using lazy approach..." << std::endl;
 
   propositional_variable_instantiation initial_state = pbes_spec.initial_state();
   atermpp::vector<pbes_equation> eqsys = pbes_spec.equations();
@@ -205,7 +205,7 @@ pbes<> do_lazy_algorithm(pbes<> pbes_spec, PbesRewriter& rewrite)
 
   std::vector< core::identifier_string > names_order;
 
-  mCRL2log(verbose) << "Retrieving PBES equations from equation system..." << std::endl;
+  mCRL2log(log::verbose) << "Retrieving PBES equations from equation system..." << std::endl;
   for (atermpp::vector<pbes_equation>::iterator eqi = eqsys.begin(); eqi != eqsys.end(); eqi++)
   {
     pbes_equations.put(eqi->variable().name(), pbes_equation_to_aterm(*eqi));
@@ -214,7 +214,7 @@ pbes<> do_lazy_algorithm(pbes<> pbes_spec, PbesRewriter& rewrite)
 
   states_todo.insert(initial_state);
 
-  mCRL2log(verbose) << "Computing BES..." << std::endl;
+  mCRL2log(log::verbose) << "Computing BES..." << std::endl;
   while (states_todo.size() != 0)
   {
     // Get the first element of the set
@@ -264,11 +264,11 @@ pbes<> do_lazy_algorithm(pbes<> pbes_spec, PbesRewriter& rewrite)
 
     if (++nr_of_equations % 1000 == 0)
     {
-      mCRL2log(verbose) << "At equation " <<  nr_of_equations << std::endl;
+      mCRL2log(log::verbose) << "At equation " <<  nr_of_equations << std::endl;
     }
   }
 
-  mCRL2log(verbose) << "Sorting result..." << std::endl;
+  mCRL2log(log::verbose) << "Sorting result..." << std::endl;
   new_equation_system = sort_names(names_order, new_equation_system);
 
   // Rewrite initial state
@@ -288,7 +288,7 @@ pbes<> do_finite_algorithm(pbes<> pbes_spec, PbesRewriter& rewrite, const mcrl2:
 {
   // Instantiate free variables in the system
   pbes_system::detail::instantiate_global_variables(pbes_spec);
-  mCRL2log(verbose) << "Using finite approach..." << std::endl;
+  mCRL2log(log::verbose) << "Using finite approach..." << std::endl;
 
   propositional_variable_instantiation initial_state = pbes_spec.initial_state();
   atermpp::vector<pbes_equation> eqsys = pbes_spec.equations();
@@ -304,7 +304,7 @@ pbes<> do_finite_algorithm(pbes<> pbes_spec, PbesRewriter& rewrite, const mcrl2:
   atermpp::table sort_enumerations(10,50);
 
   //Populate sort_enumerations with all enumerations for the finite sorts of the system
-  mCRL2log(verbose) << "Enumerating finite data sorts..." << std::endl;
+  mCRL2log(log::verbose) << "Enumerating finite data sorts..." << std::endl;
   typedef data::classic_enumerator<> enumerator_type;
   enumerator_type enumerator(data, data_rewriter);
   for (atermpp::vector<pbes_equation>::const_iterator eq_i = eqsys.begin(); eq_i != eqsys.end(); eq_i++)
@@ -326,7 +326,8 @@ pbes<> do_finite_algorithm(pbes<> pbes_spec, PbesRewriter& rewrite, const mcrl2:
           for(typename enumerator_type::iterator i=enumerator.begin(std::vector<variable>(1,x),sort_bool::true_());
                    i!=enumerator.end(); ++i)
             {
-              const data_expression d=i->find(x)->second;
+              //const data_expression d=i->find(x)->second;
+              const data_expression d=(*i)(x);
               l=push_front(l,d);
             }
           sort_enumerations.put(current_sort, l);
@@ -335,7 +336,7 @@ pbes<> do_finite_algorithm(pbes<> pbes_spec, PbesRewriter& rewrite, const mcrl2:
     }
   }
 
-  mCRL2log(verbose) << "Computing PBES without finite data sorts..." << std::endl;
+  mCRL2log(log::verbose) << "Computing PBES without finite data sorts..." << std::endl;
   for (atermpp::vector<pbes_equation>::iterator eq_i = eqsys.begin(); eq_i != eqsys.end(); eq_i++)
   {
     pbes_equation equation = *eq_i;
@@ -354,7 +355,7 @@ pbes<> do_finite_algorithm(pbes<> pbes_spec, PbesRewriter& rewrite, const mcrl2:
     instantiation_list.push_back(current_values);
 
     std::string propvar_name_string = propvar_name;
-    mCRL2log(verbose) << "Creating all possible instantiations for propositional variable " <<  propvar_name_string << "..." << std::endl;
+    mCRL2log(log::verbose) << "Creating all possible instantiations for propositional variable " <<  propvar_name_string << "..." << std::endl;
 
     for (data::variable_list::iterator p = propvar_parameters.begin(); p != propvar_parameters.end(); p++)
     {
@@ -388,7 +389,7 @@ pbes<> do_finite_algorithm(pbes<> pbes_spec, PbesRewriter& rewrite, const mcrl2:
       instantiation_list = intermediate_instantiation_list;
     }
 
-    mCRL2log(verbose) << "Computing Boolean equations for each instantiation of propositional variable " <<  propvar_name_string << "..." << std::endl;
+    mCRL2log(log::verbose) << "Computing Boolean equations for each instantiation of propositional variable " <<  propvar_name_string << "..." << std::endl;
 
     for (atermpp::vector< t_instantiations >::iterator inst_i = instantiation_list.begin(); inst_i != instantiation_list.end(); inst_i++)
     {
@@ -416,13 +417,13 @@ pbes<> do_finite_algorithm(pbes<> pbes_spec, PbesRewriter& rewrite, const mcrl2:
 
       if (++nr_of_equations % 1000 == 0)
       {
-        mCRL2log(verbose) << "At Boolean equation " <<  nr_of_equations << "" << std::endl;
+        mCRL2log(log::verbose) << "At Boolean equation " <<  nr_of_equations << "" << std::endl;
       }
 
     }
   }
 
-  mCRL2log(verbose) << "Instantiation process finished.\nNumber of Boolean equations computed: " <<  nr_of_equations << "" << std::endl;
+  mCRL2log(log::verbose) << "Instantiation process finished.\nNumber of Boolean equations computed: " <<  nr_of_equations << "" << std::endl;
 
   // rewrite initial state
   propositional_variable_instantiation new_initial_state = create_naive_propositional_variable_instantiation(initial_state, &sort_enumerations);
