@@ -17,7 +17,7 @@
 #ifndef MCRL2_PROCESS_TRAVERSER_H
 #define MCRL2_PROCESS_TRAVERSER_H
 
-#include "mcrl2/data/traverser.h"
+#include "mcrl2/lps/traverser.h"
 #include "mcrl2/lps/specification.h"
 #include "mcrl2/process/process_specification.h"
 
