@@ -27,12 +27,12 @@ static bool read_trace(Trace& t, const char* s)
 
 void test_next_action(Trace& t, const char* s)
 {
-  ATermAppl a = t.nextAction();
-  BOOST_CHECK((a != NULL) && (ATgetType((ATerm) a) == AT_APPL));
-  if ((a != NULL) && (ATgetType((ATerm) a) == AT_APPL))
+  mcrl2::lps::multi_action a = t.nextAction();
+  BOOST_CHECK(a != mcrl2::lps::multi_action());
+  if (a != mcrl2::lps::multi_action())
   {
-    BOOST_CHECK(ATgetArity(ATgetAFun(a)) == 0);
-    BOOST_CHECK(!strcmp(ATgetName(ATgetAFun(a)),s));
+    BOOST_CHECK(a.actions().size() == 1);
+    BOOST_CHECK(pp(a)==s);
   }
 }
 
@@ -44,7 +44,7 @@ int test_main(int argc, char** argv)
 
   BOOST_REQUIRE(read_trace(t,trace_str));
 
-  BOOST_REQUIRE(t.getLength() == 3);
+  BOOST_REQUIRE(t.number_of_actions() == 3);
 
   test_next_action(t,"a");
   test_next_action(t,"b(1,true)");
