@@ -14,10 +14,13 @@
 //This file describes the functions that can be used for the internal ATerm
 //structure.
 
-#include <aterm2.h>
+#include "mcrl2/aterm/aterm2.h"
 #include <string.h>
 #include <ctype.h>
 
+// XXX Remove
+using namespace aterm;
+
 //Global precondition: the ATerm library has been initialised
 
 //--- begin generated code
@@ -25,7 +28,7 @@
 inline
 AFun initAFunAltStat(AFun& f)
 {
-  f = ATmakeAFun("AltStat", 2, ATfalse);
+  f = ATmakeAFun("AltStat", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -47,7 +50,7 @@ bool gsIsAltStat(ATermAppl Term)
 inline
 AFun initAFunAssignmentGGStat(AFun& f)
 {
-  f = ATmakeAFun("AssignmentGGStat", 4, ATfalse);
+  f = ATmakeAFun("AssignmentGGStat", 4, false);
   ATprotectAFun(f);
   return f;
 }
@@ -69,7 +72,7 @@ bool gsIsAssignmentGGStat(ATermAppl Term)
 inline
 AFun initAFunAssignmentStat(AFun& f)
 {
-  f = ATmakeAFun("AssignmentStat", 4, ATfalse);
+  f = ATmakeAFun("AssignmentStat", 4, false);
   ATprotectAFun(f);
   return f;
 }
@@ -91,7 +94,7 @@ bool gsIsAssignmentStat(ATermAppl Term)
 inline
 AFun initAFunBinaryExpression(AFun& f)
 {
-  f = ATmakeAFun("BinaryExpression", 4, ATfalse);
+  f = ATmakeAFun("BinaryExpression", 4, false);
   ATprotectAFun(f);
   return f;
 }
@@ -113,7 +116,7 @@ bool gsIsBinaryExpression(ATermAppl Term)
 inline
 AFun initAFunBinaryListExpression(AFun& f)
 {
-  f = ATmakeAFun("BinaryListExpression", 4, ATfalse);
+  f = ATmakeAFun("BinaryListExpression", 4, false);
   ATprotectAFun(f);
   return f;
 }
@@ -135,7 +138,7 @@ bool gsIsBinaryListExpression(ATermAppl Term)
 inline
 AFun initAFunBinarySetExpression(AFun& f)
 {
-  f = ATmakeAFun("BinarySetExpression", 4, ATfalse);
+  f = ATmakeAFun("BinarySetExpression", 4, false);
   ATprotectAFun(f);
   return f;
 }
@@ -157,7 +160,7 @@ bool gsIsBinarySetExpression(ATermAppl Term)
 inline
 AFun initAFunChanDecl(AFun& f)
 {
-  f = ATmakeAFun("ChanDecl", 1, ATfalse);
+  f = ATmakeAFun("ChanDecl", 1, false);
   ATprotectAFun(f);
   return f;
 }
@@ -179,7 +182,7 @@ bool gsIsChanDecl(ATermAppl Term)
 inline
 AFun initAFunChanSpec(AFun& f)
 {
-  f = ATmakeAFun("ChanSpec", 1, ATfalse);
+  f = ATmakeAFun("ChanSpec", 1, false);
   ATprotectAFun(f);
   return f;
 }
@@ -201,7 +204,7 @@ bool gsIsChanSpec(ATermAppl Term)
 inline
 AFun initAFunChannelID(AFun& f)
 {
-  f = ATmakeAFun("ChannelID", 2, ATfalse);
+  f = ATmakeAFun("ChannelID", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -223,7 +226,7 @@ bool gsIsChannelID(ATermAppl Term)
 inline
 AFun initAFunChannelTypedID(AFun& f)
 {
-  f = ATmakeAFun("ChannelTypedID", 3, ATfalse);
+  f = ATmakeAFun("ChannelTypedID", 3, false);
   ATprotectAFun(f);
   return f;
 }
@@ -245,7 +248,7 @@ bool gsIsChannelTypedID(ATermAppl Term)
 inline
 AFun initAFunChiSpec(AFun& f)
 {
-  f = ATmakeAFun("ChiSpec", 2, ATfalse);
+  f = ATmakeAFun("ChiSpec", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -267,7 +270,7 @@ bool gsIsChiSpec(ATermAppl Term)
 inline
 AFun initAFunDataVarExprID(AFun& f)
 {
-  f = ATmakeAFun("DataVarExprID", 2, ATfalse);
+  f = ATmakeAFun("DataVarExprID", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -289,7 +292,7 @@ bool gsIsDataVarExprID(ATermAppl Term)
 inline
 AFun initAFunDataVarID(AFun& f)
 {
-  f = ATmakeAFun("DataVarID", 2, ATfalse);
+  f = ATmakeAFun("DataVarID", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -311,7 +314,7 @@ bool gsIsDataVarID(ATermAppl Term)
 inline
 AFun initAFunDelta(AFun& f)
 {
-  f = ATmakeAFun("Delta", 0, ATfalse);
+  f = ATmakeAFun("Delta", 0, false);
   ATprotectAFun(f);
   return f;
 }
@@ -333,7 +336,7 @@ bool gsIsDelta(ATermAppl Term)
 inline
 AFun initAFunDeltaStat(AFun& f)
 {
-  f = ATmakeAFun("DeltaStat", 2, ATfalse);
+  f = ATmakeAFun("DeltaStat", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -355,7 +358,7 @@ bool gsIsDeltaStat(ATermAppl Term)
 inline
 AFun initAFunExpression(AFun& f)
 {
-  f = ATmakeAFun("Expression", 2, ATfalse);
+  f = ATmakeAFun("Expression", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -377,7 +380,7 @@ bool gsIsExpression(ATermAppl Term)
 inline
 AFun initAFunFunction(AFun& f)
 {
-  f = ATmakeAFun("Function", 3, ATfalse);
+  f = ATmakeAFun("Function", 3, false);
   ATprotectAFun(f);
   return f;
 }
@@ -399,7 +402,7 @@ bool gsIsFunction(ATermAppl Term)
 inline
 AFun initAFunFunction2(AFun& f)
 {
-  f = ATmakeAFun("Function2", 4, ATfalse);
+  f = ATmakeAFun("Function2", 4, false);
   ATprotectAFun(f);
   return f;
 }
@@ -421,7 +424,7 @@ bool gsIsFunction2(ATermAppl Term)
 inline
 AFun initAFunGuardedStarStat(AFun& f)
 {
-  f = ATmakeAFun("GuardedStarStat", 2, ATfalse);
+  f = ATmakeAFun("GuardedStarStat", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -443,7 +446,7 @@ bool gsIsGuardedStarStat(ATermAppl Term)
 inline
 AFun initAFunInstantiation(AFun& f)
 {
-  f = ATmakeAFun("Instantiation", 2, ATfalse);
+  f = ATmakeAFun("Instantiation", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -465,7 +468,7 @@ bool gsIsInstantiation(ATermAppl Term)
 inline
 AFun initAFunListLiteral(AFun& f)
 {
-  f = ATmakeAFun("ListLiteral", 2, ATfalse);
+  f = ATmakeAFun("ListLiteral", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -487,7 +490,7 @@ bool gsIsListLiteral(ATermAppl Term)
 inline
 AFun initAFunListType(AFun& f)
 {
-  f = ATmakeAFun("ListType", 1, ATfalse);
+  f = ATmakeAFun("ListType", 1, false);
   ATprotectAFun(f);
   return f;
 }
@@ -509,7 +512,7 @@ bool gsIsListType(ATermAppl Term)
 inline
 AFun initAFunModelDef(AFun& f)
 {
-  f = ATmakeAFun("ModelDef", 2, ATfalse);
+  f = ATmakeAFun("ModelDef", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -531,7 +534,7 @@ bool gsIsModelDef(ATermAppl Term)
 inline
 AFun initAFunModelSpec(AFun& f)
 {
-  f = ATmakeAFun("ModelSpec", 2, ATfalse);
+  f = ATmakeAFun("ModelSpec", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -553,7 +556,7 @@ bool gsIsModelSpec(ATermAppl Term)
 inline
 AFun initAFunNil(AFun& f)
 {
-  f = ATmakeAFun("Nil", 0, ATfalse);
+  f = ATmakeAFun("Nil", 0, false);
   ATprotectAFun(f);
   return f;
 }
@@ -575,7 +578,7 @@ bool gsIsNil(ATermAppl Term)
 inline
 AFun initAFunOptChannel(AFun& f)
 {
-  f = ATmakeAFun("OptChannel", 0, ATfalse);
+  f = ATmakeAFun("OptChannel", 0, false);
   ATprotectAFun(f);
   return f;
 }
@@ -597,7 +600,7 @@ bool gsIsOptChannel(ATermAppl Term)
 inline
 AFun initAFunOptGuard(AFun& f)
 {
-  f = ATmakeAFun("OptGuard", 0, ATfalse);
+  f = ATmakeAFun("OptGuard", 0, false);
   ATprotectAFun(f);
   return f;
 }
@@ -619,7 +622,7 @@ bool gsIsOptGuard(ATermAppl Term)
 inline
 AFun initAFunParStat(AFun& f)
 {
-  f = ATmakeAFun("ParStat", 2, ATfalse);
+  f = ATmakeAFun("ParStat", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -641,7 +644,7 @@ bool gsIsParStat(ATermAppl Term)
 inline
 AFun initAFunParenthesisedStat(AFun& f)
 {
-  f = ATmakeAFun("ParenthesisedStat", 1, ATfalse);
+  f = ATmakeAFun("ParenthesisedStat", 1, false);
   ATprotectAFun(f);
   return f;
 }
@@ -663,7 +666,7 @@ bool gsIsParenthesisedStat(ATermAppl Term)
 inline
 AFun initAFunProcDecl(AFun& f)
 {
-  f = ATmakeAFun("ProcDecl", 1, ATfalse);
+  f = ATmakeAFun("ProcDecl", 1, false);
   ATprotectAFun(f);
   return f;
 }
@@ -685,7 +688,7 @@ bool gsIsProcDecl(ATermAppl Term)
 inline
 AFun initAFunProcDef(AFun& f)
 {
-  f = ATmakeAFun("ProcDef", 3, ATfalse);
+  f = ATmakeAFun("ProcDef", 3, false);
   ATprotectAFun(f);
   return f;
 }
@@ -707,7 +710,7 @@ bool gsIsProcDef(ATermAppl Term)
 inline
 AFun initAFunProcSpec(AFun& f)
 {
-  f = ATmakeAFun("ProcSpec", 2, ATfalse);
+  f = ATmakeAFun("ProcSpec", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -729,7 +732,7 @@ bool gsIsProcSpec(ATermAppl Term)
 inline
 AFun initAFunRecv(AFun& f)
 {
-  f = ATmakeAFun("Recv", 0, ATfalse);
+  f = ATmakeAFun("Recv", 0, false);
   ATprotectAFun(f);
   return f;
 }
@@ -751,7 +754,7 @@ bool gsIsRecv(ATermAppl Term)
 inline
 AFun initAFunRecvStat(AFun& f)
 {
-  f = ATmakeAFun("RecvStat", 4, ATfalse);
+  f = ATmakeAFun("RecvStat", 4, false);
   ATprotectAFun(f);
   return f;
 }
@@ -773,7 +776,7 @@ bool gsIsRecvStat(ATermAppl Term)
 inline
 AFun initAFunSend(AFun& f)
 {
-  f = ATmakeAFun("Send", 0, ATfalse);
+  f = ATmakeAFun("Send", 0, false);
   ATprotectAFun(f);
   return f;
 }
@@ -795,7 +798,7 @@ bool gsIsSend(ATermAppl Term)
 inline
 AFun initAFunSendStat(AFun& f)
 {
-  f = ATmakeAFun("SendStat", 4, ATfalse);
+  f = ATmakeAFun("SendStat", 4, false);
   ATprotectAFun(f);
   return f;
 }
@@ -817,7 +820,7 @@ bool gsIsSendStat(ATermAppl Term)
 inline
 AFun initAFunSepStat(AFun& f)
 {
-  f = ATmakeAFun("SepStat", 2, ATfalse);
+  f = ATmakeAFun("SepStat", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -839,7 +842,7 @@ bool gsIsSepStat(ATermAppl Term)
 inline
 AFun initAFunSetLiteral(AFun& f)
 {
-  f = ATmakeAFun("SetLiteral", 2, ATfalse);
+  f = ATmakeAFun("SetLiteral", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -861,7 +864,7 @@ bool gsIsSetLiteral(ATermAppl Term)
 inline
 AFun initAFunSetType(AFun& f)
 {
-  f = ATmakeAFun("SetType", 1, ATfalse);
+  f = ATmakeAFun("SetType", 1, false);
   ATprotectAFun(f);
   return f;
 }
@@ -883,7 +886,7 @@ bool gsIsSetType(ATermAppl Term)
 inline
 AFun initAFunSkip(AFun& f)
 {
-  f = ATmakeAFun("Skip", 0, ATfalse);
+  f = ATmakeAFun("Skip", 0, false);
   ATprotectAFun(f);
   return f;
 }
@@ -905,7 +908,7 @@ bool gsIsSkip(ATermAppl Term)
 inline
 AFun initAFunSkipStat(AFun& f)
 {
-  f = ATmakeAFun("SkipStat", 3, ATfalse);
+  f = ATmakeAFun("SkipStat", 3, false);
   ATprotectAFun(f);
   return f;
 }
@@ -927,7 +930,7 @@ bool gsIsSkipStat(ATermAppl Term)
 inline
 AFun initAFunStarStat(AFun& f)
 {
-  f = ATmakeAFun("StarStat", 1, ATfalse);
+  f = ATmakeAFun("StarStat", 1, false);
   ATprotectAFun(f);
   return f;
 }
@@ -949,7 +952,7 @@ bool gsIsStarStat(ATermAppl Term)
 inline
 AFun initAFunTupleDot(AFun& f)
 {
-  f = ATmakeAFun("TupleDot", 3, ATfalse);
+  f = ATmakeAFun("TupleDot", 3, false);
   ATprotectAFun(f);
   return f;
 }
@@ -971,7 +974,7 @@ bool gsIsTupleDot(ATermAppl Term)
 inline
 AFun initAFunTupleLiteral(AFun& f)
 {
-  f = ATmakeAFun("TupleLiteral", 2, ATfalse);
+  f = ATmakeAFun("TupleLiteral", 2, false);
   ATprotectAFun(f);
   return f;
 }
@@ -993,7 +996,7 @@ bool gsIsTupleLiteral(ATermAppl Term)
 inline
 AFun initAFunTupleType(AFun& f)
 {
-  f = ATmakeAFun("TupleType", 1, ATfalse);
+  f = ATmakeAFun("TupleType", 1, false);
   ATprotectAFun(f);
   return f;
 }
@@ -1015,7 +1018,7 @@ bool gsIsTupleType(ATermAppl Term)
 inline
 AFun initAFunType(AFun& f)
 {
-  f = ATmakeAFun("Type", 1, ATfalse);
+  f = ATmakeAFun("Type", 1, false);
   ATprotectAFun(f);
   return f;
 }
@@ -1037,7 +1040,7 @@ bool gsIsType(ATermAppl Term)
 inline
 AFun initAFunUnaryExpression(AFun& f)
 {
-  f = ATmakeAFun("UnaryExpression", 3, ATfalse);
+  f = ATmakeAFun("UnaryExpression", 3, false);
   ATprotectAFun(f);
   return f;
 }
@@ -1059,7 +1062,7 @@ bool gsIsUnaryExpression(ATermAppl Term)
 inline
 AFun initAFunVarDecl(AFun& f)
 {
-  f = ATmakeAFun("VarDecl", 1, ATfalse);
+  f = ATmakeAFun("VarDecl", 1, false);
   ATprotectAFun(f);
   return f;
 }
@@ -1081,7 +1084,7 @@ bool gsIsVarDecl(ATermAppl Term)
 inline
 AFun initAFunVarSpec(AFun& f)
 {
-  f = ATmakeAFun("VarSpec", 1, ATfalse);
+  f = ATmakeAFun("VarSpec", 1, false);
   ATprotectAFun(f);
   return f;
 }
@@ -1404,7 +1407,7 @@ ATermAppl gsString2ATermAppl(const char* s)
 {
   if (s != NULL)
   {
-    return ATmakeAppl0(ATmakeAFun(s, 0, ATtrue));
+    return ATmakeAppl0(ATmakeAFun(s, 0, true));
   }
   else
   {
@@ -1417,7 +1420,7 @@ bool gsIsString(ATermAppl term)
 //Ret: term is a quoted constant
 {
   AFun head = ATgetAFun(term);
-  return ((ATgetArity(head) == 0) && (ATisQuoted(head) == ATtrue));
+  return ((ATgetArity(head) == 0) && (ATisQuoted(head) == true));
 }
 
 inline
