@@ -12,8 +12,9 @@
 #include <cstring>
 #include <cmath>
 #include <iostream>
-#include <aterm2.h>
-#include "mcrl2/core/messaging.h"
+#include "mcrl2/aterm/aterm2.h"
+#include "mcrl2/utilities/logger.h"
+#include "mcrl2/utilities/text_utility.h"
 #include <cstdio>
 #include <vector>
 #include <map>
@@ -32,6 +33,9 @@
 #define RPC RecProcessChannels
 #define RC  RecChannel
 
+// XXX Remove
+using namespace aterm;
+
 struct t_options
 {
   std::string infilename;
@@ -117,20 +121,6 @@ typedef struct
   std::string Type;
 } RecChannel;
 
-template <class T>
-inline std::string to_string(const T& t)
-{
-  std::stringstream ss;
-  ss << t;
-  return ss.str();
-}
-
-template < >
-inline std::string to_string(const std::string& t)
-{
-  return t;
-}
-
 class CAsttransform
 {
   public:
