@@ -39,7 +39,7 @@ void grape::mcrl2gen::init_mcrl2libs(int p_argc, char** p_argv)
 {
   // initialize ATerm library and set verbose messaging
   MCRL2_ATERMPP_INIT(p_argc, p_argv);
-  mcrl2_logger::set_reporting_level(log_verbose);
+  mcrl2::log::mcrl2_logger::set_reporting_level(mcrl2::log::verbose);
 }
 
 void grape::mcrl2gen::verbose_actions(list_of_action& p_actions)
