@@ -11,16 +11,13 @@
 
 #include "boost.hpp" // precompiled headers
 
-#include "mcrl2/lps/binary.h"
+#include "mcrl2/lps/tools.h"
 
 #include "mcrl2/utilities/input_output_tool.h"
 #include "mcrl2/utilities/rewriter_tool.h"
 #include "mcrl2/utilities/mcrl2_gui_tool.h"
 #include "mcrl2/atermpp/aterm_init.h"
 
-
-using namespace std;
-using namespace mcrl2;
 using namespace mcrl2::utilities;
 using namespace mcrl2::utilities::tools;
 
@@ -45,13 +42,7 @@ class binary_tool: public rewriter_tool<input_output_tool>
 
     bool run()
     {
-      lps::specification spec;
-      spec.load(m_input_filename);
-      data::rewriter r(create_rewriter(spec.data()));
-
-      lps::binary_algorithm<data::rewriter>(spec, r).run();
-      spec.save(m_output_filename);
-
+      mcrl2::lps::lpsbinary(m_input_filename, m_output_filename);
       return true;
     }
 
