@@ -14,7 +14,7 @@
 
 #include "mcrl2/utilities/logger.h"
 #include "mcrl2/data/find.h"
-#include "mcrl2/data/postfix_identifier_generator.h"
+#include "mcrl2/data/set_identifier_generator.h"
 #include "mcrl2/data/standard_utility.h"
 #include "mcrl2/data/replace.h"
 
@@ -23,11 +23,11 @@
 #include "realelm.h"
 #include "linear_inequalities.h"
 
-using namespace atermpp;
 using namespace mcrl2;
 using namespace mcrl2::core;
 using namespace mcrl2::data;
 using namespace mcrl2::lps;
+using namespace mcrl2::log;
 
 
 static data_expression negate_inequality(const data_expression e)
@@ -98,45 +98,11 @@ variable_list get_nonreal_variables(const variable_list& l)
 
 static data::function_symbol& negate_function_symbol(const sort_expression s)
 {
-  static data::function_symbol f = initialise_static_expression(f,data::function_symbol("negate",data::make_function_sort(s,s)));
+  static data::function_symbol f = core::detail::initialise_static_expression(f,data::function_symbol("negate",data::make_function_sort(s,s)));
   assert(data::make_function_sort(s,s)==f.sort()); // Protect against using f for other sorts than sort comp.
   return f;
 }
 
-/// \brief Returns a list of all real expressions in l
-/// \param l a list of data expressions
-/// \ret The list of all e in l such that e.sort() == real()
-static inline
-data_expression_list get_real_expressions(const data_expression_list& l)
-{
-  data_expression_list r;
-  for (data_expression_list::const_iterator i = l.begin(); i != l.end(); ++i)
-  {
-    if (i->sort() == sort_real::real_())
-    {
-      r = push_front(r, *i);
-    }
-  }
-  return r;
-}
-
-/// \brief Returns a list of all nonreal expressions in l
-/// \param l a list of data expressions
-/// \ret The list of all e in l such that e.sort() != real()
-static inline
-data_expression_list get_nonreal_expressions(const data_expression_list& l)
-{
-  data_expression_list r;
-  for (data_expression_list::const_iterator i = l.begin(); i != l.end(); ++i)
-  {
-    if (i->sort() != sort_real::real_())
-    {
-      r = push_front(r, *i);
-    }
-  }
-  return r;
-}
-
 /// \brief Returns a list of all real assignments in l
 /// \param l a list of data assignments
 /// \ret The list of all x := e in l such that x.sort() == e.sort() == real()
@@ -209,7 +175,7 @@ static data_expression else_part(const data_expression e)
 /// \brief Splits a condition in expressions ranging over reals and the others
 /// \details Conceptually, the condition is first transformed to conjunctive
 ///          normalform. For each conjunct, there will be an entry in both
-///          resulting vectors, where the real conditions are in "real_conditions",
+///          resulting std::vectors, where the real conditions are in "real_conditions",
 ///          and the others in non_real_conditions. If there are conjuncts with
 ///          both real and non-real variables an exception is thrown. If negate
 ///          is true the result will be negated.
@@ -225,7 +191,7 @@ static void split_condition(
   atermpp::vector < data_expression_list > &non_real_conditions,
   const bool negate=false)
 {
-  // std::cerr << "Split condition " << data::pp(e) << "\n";
+  // mCRL2log(debug) << "Split condition " << data::pp(e) << "\n";
   real_conditions.clear();
   non_real_conditions.clear();
 
@@ -346,8 +312,8 @@ static void normalize_specification(
   for (lps::deprecated::summand_list::const_iterator i = smds.begin(); i != smds.end(); ++i)
   {
     atermpp::vector <data_expression_list> real_conditions, non_real_conditions;
-    // std::cerr << "SUMMANDNORM: " << lps::pp(*i) << "\n";
-    // std::cerr << "Condition in: " << data::pp(i->condition()) << "\n";
+    // mCRL2log(debug) << "SUMMANDNORM: " << lps::pp(*i) << "\n";
+    // mCRL2log(debug) << "Condition in: " << data::pp(i->condition()) << "\n";
     split_condition(i->condition(),real_conditions,non_real_conditions);
 
     for (atermpp::vector <data_expression_list>::const_iterator
@@ -360,7 +326,7 @@ static void normalize_specification(
       {
         t=set_condition(t,c);
 
-        vector < linear_inequality > inequalities;
+        std::vector < linear_inequality > inequalities;
         // Collect all real conditions from the condition from this summand and put them
         // into inequalities.
         for (data_expression_list::const_iterator k=j_r->begin(); k!=j_r->end(); k++)
@@ -372,10 +338,10 @@ static void normalize_specification(
         // next state. We can apply Fourier-Motzkin to eliminate these variables from
         // this sum operator and the condition.
 
-        // std::cerr << "REALPARS " << data::pp(i->next_state(real_parameters)) << "\n";
+        // mCRL2log(debug) << "REALPARS " << data::pp(i->next_state(real_parameters)) << "\n";
         const std::set < variable> s1=data::find_variables(i->next_state(real_parameters));
         // for(std::set < variable>::const_iterator k=s1.begin(); k!=s1.end(); ++k)
-        // { std::cerr << "VAR " << data::pp(*k) << "\n";
+        // { mCRL2log(debug) << "VAR " << data::pp(*k) << "\n";
 //
 //
         // }
@@ -387,7 +353,7 @@ static void normalize_specification(
              k!=original_real_sum_variables.end(); ++k)
         {
 
-          // std::cerr << "Treat " << data::pp(*k) << "\n";
+          // mCRL2log(debug) << "Treat " << data::pp(*k) << "\n";
           if (s1.count(*k)==0)
           {
             // The variable does not occur in the parameters. We can eliminate it using Fourier-Motzkin
@@ -399,7 +365,7 @@ static void normalize_specification(
           }
         }
 
-        vector < linear_inequality > new_inequalities;
+        std::vector < linear_inequality > new_inequalities;
         fourier_motzkin(inequalities,
                         eliminatable_real_sum_variables.begin(),
                         eliminatable_real_sum_variables.end(),
@@ -410,7 +376,7 @@ static void normalize_specification(
 
         if ((inequalities.size()>0) && (inequalities.front().is_false(r)))
         {
-          //  std::cerr << "INCONSISTENT \n";
+          //  mCRL2log(debug) << "INCONSISTENT \n";
         }
         else
         {
@@ -454,30 +420,6 @@ static void normalize_specification(
   //return s;
 }
 
-/// \brief Determine the inequalities ranging over real numbers in a data expression.
-/// \param e A data expression
-/// \param inequalities A list of inequalities
-/// \post inequalities contains all inequalities ranging over real numbers in e.
-static
-void determine_real_inequalities(
-  const data_expression& e,
-  vector < linear_inequality > &inequalities,
-  const rewriter& r)
-{
-  // std::cerr << "Real inequalities in" << data::pp(e) << "\n";
-  if (sort_bool::is_and_application(e))
-  {
-    determine_real_inequalities(application(e).left(), inequalities,r);
-    determine_real_inequalities(application(e).right(), inequalities,r);
-  }
-  else if (is_inequality(e) && (application(e).right().sort() == sort_real::real_()))
-  {
-    inequalities.push_back(linear_inequality(e,r));
-  }
-  // std::cerr << "Real inequalities out" << pp_vector(inequalities) << "\n";
-  //else Do nothing, as it is not an expression on reals
-}
-
 /// \brief Add postponed inequalities to variable context
 /// \param inequalities_to_add The pairs of inequalities to be added.
 /// \param context A variable context
@@ -539,8 +481,8 @@ static void add_inequalities_to_context_postponed(
 {
   assert(inequalities_to_add_lhs.size()==inequalities_to_add_lhs_size.size() &&
          inequalities_to_add_lhs.size() ==inequalities_to_add_rhs.size());
-  // std::cerr << "Inequalities to add: " << pp_vector(inequalities_to_add) << "\n";
-  for (vector < linear_inequality > ::iterator i = l.begin(); i != l.end();)
+  // mCRL2log(debug) << "Inequalities to add: " << pp_vector(inequalities_to_add) << "\n";
+  for (std::vector < linear_inequality > ::iterator i = l.begin(); i != l.end();)
   {
     data_expression left;
     data_expression right;
@@ -576,7 +518,7 @@ static void add_inequalities_to_context_postponed(
           }
           else
           {
-            // std::cerr << "Reserved to be added <" << data::pp(left) << "," << data::pp(right) << "\n";
+            // mCRL2log(debug) << "Reserved to be added <" << data::pp(left) << "," << data::pp(right) << "\n";
             inequalities_to_add_lhs_size.push_back(i->lhs().size()); // store the number of variables at the lhs.
             inequalities_to_add_lhs.push_back(left);
             inequalities_to_add_rhs.push_back(right);
@@ -611,7 +553,7 @@ lps::deprecated::summand generate_summand(summand_information& summand_info,
                          const comp_struct& cs,
                          const bool is_may_summand=false)
 {
-  // std::cerr << "SUMMAND " << lps::pp(summand_info.get_summand()) << "\nCOND " << data::pp(new_condition) << "\n";
+  // mCRL2log(debug) << "SUMMAND " << lps::pp(summand_info.get_summand()) << "\nCOND " << data::pp(new_condition) << "\n";
   static atermpp::vector < sort_expression_list > protect_against_garbage_collect;
   static std::map < std::pair < std::string, sort_expression_list >, std::string> action_label_map;
   // Used to recall which may actions labels have been
@@ -627,13 +569,13 @@ lps::deprecated::summand generate_summand(summand_information& summand_info,
     data_expression substituted_lowerbound=
        data::replace_free_variables(c_complete->get_lowerbound(),summand_info.get_summand_real_nextstate_map());
     data_expression substituted_upperbound=
-       data::replace_free_variables(c_complete->get_upperbound(),summand_info.get_summand_real_nextstate_map()); 
-    // std::cerr << "Lower Upper " << data::pp(substituted_lowerbound) << "  " << data::pp(substituted_upperbound) << "\n";
+       data::replace_free_variables(c_complete->get_upperbound(),summand_info.get_summand_real_nextstate_map());
+    // mCRL2log(debug) << "Lower Upper " << data::pp(substituted_lowerbound) << "  " << data::pp(substituted_upperbound) << "\n";
     linear_inequality e(substituted_lowerbound,substituted_upperbound,linear_inequality::less,r);
-    // std::cerr << "INequality: " << string(e) << "\n";
+    // mCRL2log(debug) << "INequality: " << string(e) << "\n";
     data_expression t,u;
     const bool negate=e.typical_pair(t,u,r);  // XXXXXXXXX       This can negate inequality, causing trouble.
-    // std::cerr << "Typical pair " << data::pp(t) << "  " << data::pp(u) << "   " << negate << "\n";
+    // mCRL2log(debug) << "Typical pair " << data::pp(t) << "  " << data::pp(u) << "   " << negate << "\n";
 
     bool success(false);
     /* First check whether the pair < t,u >
@@ -666,10 +608,10 @@ lps::deprecated::summand generate_summand(summand_information& summand_info,
                                       substituted_lowerbound,
                                       substituted_upperbound,
                                       linear_inequality::less_eq,r));
-      // std::cerr << "Nextstate_condition " << pp_vector(nextstate_condition) << "\n";
+      // mCRL2log(debug) << "Nextstate_condition " << pp_vector(nextstate_condition) << "\n";
       if (is_inconsistent(nextstate_condition,r))
       {
-        // std::cerr << "ASSIGNMENT: " << c_complete->get_variable() << " := " << data_expression(cs.larger()) << "\n";
+        // mCRL2log(debug) << "ASSIGNMENT: " << c_complete->get_variable() << " := " << data_expression(cs.larger()) << "\n";
         nextstate=push_front(nextstate,assignment(c_complete->get_variable(),data_expression(cs.larger())));
       }
       else
@@ -725,8 +667,6 @@ lps::deprecated::summand generate_summand(summand_information& summand_info,
   lps::deprecated::summand result = lps::deprecated::summand(get_nonreal_variables(s.summation_variables()),
                            new_condition, s.is_delta(), new_actions, nextstate);
 
-  // mCRL2log(debug) << "Generated summand " << core::pp(result) << std::endl;
-
   return result;
 }
 
@@ -737,6 +677,7 @@ lps::deprecated::summand generate_summand(summand_information& summand_info,
 /// \param r A rewriter
 /// \ret A process initialisation in which all assignments to real variables
 ///      have been replaced with an initialization for each variable in context.
+static
 assignment_list determine_process_initialization(
   const assignment_list& initialization,
   context_type& context,
@@ -797,22 +738,22 @@ specification realelm(specification s, int max_iterations, const rewriter& r)
   ds.add_alias(alias(c.basic_sort_name(),c));
   ds.add_mapping(negate_function_symbol(c.sort()));
   ds.add_equation(data_equation(  // negate(larger)=smaller;
-                    vector <variable>(),
+                    std::vector <variable>(),
                     sort_bool::true_(),
                     make_application(negate_function_symbol(c.sort()),c.larger()),
                     c.smaller()));
   ds.add_equation(data_equation(  // negate(smaller)=larger;
-                    vector <variable>(),
+                    std::vector <variable>(),
                     sort_bool::true_(),
                     make_application(negate_function_symbol(c.sort()),c.smaller()),
                     c.larger()));
   ds.add_equation(data_equation(  // negate(equal)=equal;
-                    vector <variable>(),
+                    std::vector <variable>(),
                     sort_bool::true_(),
                     make_application(negate_function_symbol(c.sort()),c.equal()),
                     c.equal()));
   variable v("x",c.sort());
-  vector <variable> vars;
+  std::vector <variable> vars;
   vars.push_back(v);
   ds.add_equation(data_equation(  // negate(negate(x))=x;
                     vars,
@@ -821,15 +762,15 @@ specification realelm(specification s, int max_iterations, const rewriter& r)
                     v));
 
   s.data() = ds;
-  postfix_identifier_generator variable_generator("");
+  set_identifier_generator variable_generator;
   variable_generator.add_identifiers(lps::find_identifiers((s)));
   linear_process lps=s.process();
   const variable_list real_parameters = get_real_variables(lps.process_parameters());
   const variable_list nonreal_parameters = get_nonreal_variables(lps.process_parameters());
   std::vector < summand_information > summand_info;
-  // std::cerr << "Normalize specification in\n";
+  // mCRL2log(debug) << "Normalize specification in\n";
   normalize_specification(s, real_parameters, r, summand_info);
-  // std::cerr << "Normalize specification out\n";
+  // mCRL2log(debug) << "Normalize specification out\n";
 
   context_type context; // Contains introduced variables
 
@@ -848,7 +789,7 @@ specification realelm(specification s, int max_iterations, const rewriter& r)
     for (std::vector < summand_information >::iterator i = summand_info.begin();
          i != summand_info.end(); ++i)
     {
-      // std::cerr << "SUMMAND_IN " << lps::pp(i->get_summand()) << "\n" ;
+      // mCRL2log(debug) << "SUMMAND_IN " << lps::pp(i->get_summand()) << "\n" ;
 
       // First calculate the newly introduced variables xi for which the next_state value is not yet known.
       // get , by only looking at variables that
@@ -863,11 +804,11 @@ specification realelm(specification s, int max_iterations, const rewriter& r)
 
         variable_list sumvars= i->get_real_summation_variables();
 
-        // std::cerr << "SUMVARS " << data::pp(sumvars) << "\n" ;
+        // mCRL2log(debug) << "SUMVARS " << data::pp(sumvars) << "\n" ;
         std::vector < linear_inequality > condition2;
         remove_redundant_inequalities(*nextstate_combination,condition2,r);
-        // std::cerr << "CONDITION IN" << pp_vector(*nextstate_combination) << "\n" ;
-        // std::cerr << "REMOVING REDUNDANT INEQUALITIES: FROM1 " << nextstate_combination->size() << " TO " << condition2.size() << "\n";
+        // mCRL2log(debug) << "CONDITION IN" << pp_vector(*nextstate_combination) << "\n" ;
+        // mCRL2log(debug) << "REMOVING REDUNDANT INEQUALITIES: FROM1 " << nextstate_combination->size() << " TO " << condition2.size() << "\n";
         *nextstate_combination=condition2;
 
         std::vector < linear_inequality > condition1;
@@ -883,9 +824,9 @@ specification realelm(specification s, int max_iterations, const rewriter& r)
 
         std::vector < linear_inequality > condition3;
         remove_redundant_inequalities(condition1,condition3,r);
-        // std::cerr << "CONDITION OUT" << pp_vector(condition3) << "\n" ;
+        // mCRL2log(debug) << "CONDITION OUT" << pp_vector(condition3) << "\n" ;
 
-        // std::cerr << "REMOVING REDUNDANT INEQUALITIES: FROM2 " << condition1.size() << " TO " << condition3.size() << "\n";
+        // mCRL2log(debug) << "REMOVING REDUNDANT INEQUALITIES: FROM2 " << condition1.size() << " TO " << condition3.size() << "\n";
         if (!is_inconsistent(condition3,r))
         {
           // condition contains the inequalities over the process parameters
@@ -912,8 +853,8 @@ specification realelm(specification s, int max_iterations, const rewriter& r)
   mCRL2log(verbose) << "Generated the following variables in " <<  iteration << " iterations:" << std::endl;
   for (context_type::iterator i = context.begin(); i != context.end(); ++i)
   {
-    mCRL2log(verbose) << "< " << core::pp(atermpp::aterm_appl(i->get_lowerbound())) << ", " << core::pp(atermpp::aterm_appl(i->get_upperbound()))
-                      << " > " << core::pp(atermpp::aterm_appl(i->get_variable())) << std::endl;
+    mCRL2log(verbose) << "< " << data::pp(i->get_lowerbound()) << ", " << data::pp(i->get_upperbound())
+                      << " > " << data::pp(i->get_variable()) << std::endl;
   }
 
   if (!new_inequalities_sizes.empty())
@@ -933,7 +874,7 @@ specification realelm(specification s, int max_iterations, const rewriter& r)
   for (std::vector < summand_information >::iterator i = summand_info.begin();
        i != summand_info.end(); ++i)
   {
-    // std::cerr << "SUMMAND_IN__ " << lps::pp(i->get_summand()) << "\n";
+    // mCRL2log(debug) << "SUMMAND_IN__ " << lps::pp(i->get_summand()) << "\n";
 
     // Construct the real time condition for summand in terms of xi variables.
 
@@ -945,7 +886,7 @@ specification realelm(specification s, int max_iterations, const rewriter& r)
          nextstate_combination != i->nextstate_context_combinations_end();
          ++ nextstate_combination)
     {
-      // std::cerr << "Nextstate cond: " << pp_vector(*nextstate_combination) << "\n";
+      // mCRL2log(debug) << "Nextstate cond: " << pp_vector(*nextstate_combination) << "\n";
 
       data_expression new_condition=i->get_summand().condition();
       std::vector < linear_inequality > real_condition1;
@@ -959,11 +900,11 @@ specification realelm(specification s, int max_iterations, const rewriter& r)
       remove_redundant_inequalities(real_condition1,real_condition2,r);
 
       bool all_conditions_found=true;
-      // std::cerr << "Normalised nextstate cond: " << pp_vector(real_condition2) << "\n";
+      // mCRL2log(debug) << "Normalised nextstate cond: " << pp_vector(real_condition2) << "\n";
       for (std::vector <linear_inequality>::const_iterator j=real_condition2.begin();
            j!=real_condition2.end(); ++j)
       {
-        // std::cerr << "condition " << string(*j) << "\n";
+        // mCRL2log(debug) << "condition " << string(*j) << "\n";
         data_expression t;
         data_expression u;
         j->typical_pair(t,u,r);
@@ -1023,7 +964,7 @@ specification realelm(specification s, int max_iterations, const rewriter& r)
                                      variable_generator,
                                      c,
                                      true);
-        // std::cerr << "MAY SUMMAND_OUT: " << lps::pp(s) << "\n";
+        // mCRL2log(debug) << "MAY SUMMAND_OUT: " << lps::pp(s) << "\n";
         summands = push_front(summands, s);
       }
       else
@@ -1038,7 +979,7 @@ specification realelm(specification s, int max_iterations, const rewriter& r)
                                      variable_generator,
                                      c,
                                      false);
-        // std::cerr << "MUST SUMMAND_OUT: " << lps::pp(s) << "\n";
+        // mCRL2log(debug) << "MUST SUMMAND_OUT: " << lps::pp(s) << "\n";
         summands = push_front(summands, s);
       }
     }
