@@ -7,7 +7,6 @@
 // http://www.boost.org/LICENSE_1_0.txt)
 
 #include "wx.hpp" // precompiled headers
-#include "mcrl2/core/print.h"
 #include "mcrl2/trace/trace.h"
 #include "mcrl2/lts/lts_algorithm.h"
 #include "mcrl2/lts/lts_io.h"
@@ -306,12 +305,12 @@ bool LTS::readFromFile(const std::string& filename)
   initialState = states[mcrl2_lts.initial_state()];
   simulation->setInitialState(initialState);
 
-  for (transition_const_range r=mcrl2_lts.get_transitions() ; !r.empty(); r.advance_begin(1))
+  const std::vector<transition> &trans=mcrl2_lts.get_transitions();
+  for (std::vector<transition>::const_iterator r=trans.begin(); r!=trans.end(); ++r)
   {
-    const transition ti=r.front();
-    State* s1 = states[ti.from()];
-    State* s2 = states[ti.to()];
-    Transition* t = new Transition(s1,s2,static_cast<int>(ti.label()));
+    State* s1 = states[r->from()];
+    State* s2 = states[r->to()];
+    Transition* t = new Transition(s1,s2,static_cast<int>(r->label()));
     if (s1 != s2)
     {
       s1->addOutTransition(t);
@@ -884,6 +883,17 @@ void LTS::loadTrace(std::string const& path)
     return;
   }
 
+  try{
+  if (tr.number_of_states()==0)
+  {
+    throw mcrl2::runtime_error("The trace in " + path + " has no state information, which is currently not allowed by ltsview.\nThis typically occurs with traces generated by ltscompare, which can currently not be used in ltsview. ");
+  }
+
+  if (tr.number_of_states()>0 && !mcrl2_lts.has_state_info())
+  {
+    throw mcrl2::runtime_error("The trace in " + path + " has state information but the labelled transition system does not have it, which is currently not allowed by ltsview.");
+  }
+
   Simulation* sim = new Simulation();
   // Initialize simulation with initial state of the LTS;
   State* initState;
@@ -896,11 +906,15 @@ void LTS::loadTrace(std::string const& path)
   }
   initState = topLevel->getInitialState();
 
+  if (tr.number_of_states()>0 && tr.currentState().size()!=mcrl2_lts.state_label(initState->getID()).size())
+  {
+    throw mcrl2::runtime_error("The trace in " + path + " and the labelled transition system have state information of unequal lengths.");
+  }
   sim->setInitialState(initState);
   sim->start();
 
-  // Get the first state of the trace (as an ATermAppl)
-  ATermAppl currState = tr.currentState();
+  // Get the first state of the trace 
+  mcrl2::lps::state currState = tr.currentState();
   // Now, currState ~ initState.
   //
   // In currState, free variables can occur, instantiate this with the values
@@ -908,22 +922,18 @@ void LTS::loadTrace(std::string const& path)
   //
   // Assumption: The ith parameter in currState is equal to the ith parameter
   // in initState.
-  for (size_t i = 0; i < ATgetLength(ATgetArguments(currState)); ++i)
+  for (size_t i = 0; i < currState.size(); ++i)
   {
-
-    ATerm currVal = ATgetArgument(currState, i);
-    string value = PrintPart_CXX(currVal, ppDefault);
+    string value = mcrl2::data::pp(currState[i]);
 
     std::string paramValue = getStateParameterValueStr(initState,i);
   }
 
   // Load the rest of the trace.
 
-  while (tr.getPosition() != tr.getLength())
+  while (tr.getPosition() != tr.number_of_actions())
   {
-    std::string action = PrintPart_CXX(ATgetArgument(
-                                         ATgetArgument(tr.nextAction(),0),0),
-                                       ppDefault);
+    std::string action = pp(tr.nextAction());
 
     std::vector<Transition*> posTrans = sim->getPosTrans();
     int possibilities = 0;
@@ -958,12 +968,10 @@ void LTS::loadTrace(std::string const& path)
         State* s = posTrans[j]->getEndState();
         int match = 0;
 
-        for (size_t i = 0; i < ATgetLength(ATgetArguments(currState));
-             ++i)
+        for (size_t i = 0; i < currState.size(); ++i)
         {
 
-          std::string currVal = PrintPart_CXX(ATgetArgument(currState, i),
-                                              ppDefault);
+          std::string currVal = mcrl2::data::pp(currState[i]);
 
           std::map<std::string, std::string>::iterator it;
 
@@ -1001,6 +1009,13 @@ void LTS::loadTrace(std::string const& path)
 
   // Set simulation to the LTS
   simulation = sim;
+  }
+  catch( mcrl2::runtime_error &e )
+  {
+    std::string error = e.what();
+    mediator->reportError(error);
+    return;
+  }
 }
 
 void LTS::generateBackTrace()
