@@ -16,6 +16,7 @@
 #include <boost/algorithm/string.hpp>
 
 using namespace boost;
+using namespace mcrl2::log;
 
 
 namespace lysa
@@ -64,18 +65,26 @@ string Converter::make_proc_def(string proc_name, string process, Context& conte
   bool b = false;
   BOOST_FOREACH(mv, context.mvars)
   {
-    if (b++)
+    if (b)
     {
       proc_args << ", ";
     }
+    else
+    {
+      b = true;
+    }
     proc_args << mv.first << ": Nat";
   }
   BOOST_FOREACH(v, context.vars)
   {
-    if (b++)
+    if (b)
     {
       proc_args << ", ";
     }
+    else
+    {
+      b = true;
+    }
     proc_args << v.second << ": Value";
   }
   if (b)
@@ -85,6 +94,8 @@ string Converter::make_proc_def(string proc_name, string process, Context& conte
   proc_def += " = " + process;
   return proc_def;
 }
+
+static
 void stream_cartesian_product_args(ostringstream& process, vector<vector<size_t> >& ds, string name, string add_args, string separator=" || ")
 {
   if (ds.empty())
@@ -106,10 +117,14 @@ void stream_cartesian_product_args(ostringstream& process, vector<vector<size_t>
     bool b = false;
     while (!stop)
     {
-      if (b++)
+      if (b)
       {
         process << separator;
       }
+      else
+      {
+    	b = true;
+      }
 
       //draw proc call
       {
@@ -203,10 +218,14 @@ string Converter::to_mcrl2(IndexedParallel& ipar, Context context)
     bool bb = false;
     BOOST_FOREACH(v, context.vars)
     {
-      if (bb++)
+      if (bb)
       {
         add_args += ", ";
       }
+      else
+      {
+    	bb = true;
+      }
       add_args += v.second;
     }
 
@@ -268,10 +287,14 @@ string Converter::to_mcrl2(OrdinaryParallel& opar, Context context)
     bool bb = false;
     BOOST_FOREACH(string proc_name, proc_names)
     {
-      if (bb++)
+      if (bb)
       {
         process << " || ";
       }
+      else
+      {
+    	  bb = true;
+      }
       pair<string, string> mv, v;
       ostringstream proc_args;
       process << proc_name;
@@ -280,19 +303,27 @@ string Converter::to_mcrl2(OrdinaryParallel& opar, Context context)
       bool b = false;
       BOOST_FOREACH(mv, context.mvars)
       {
-        if (b++)
+        if (b)
         {
           proc_args << ", ";
         }
+        else
+        {
+          b = true;
+        }
         proc_args << mv.first;
       }
       //pass vars
       BOOST_FOREACH(v, context.vars)
       {
-        if (b++)
+        if (b)
         {
           proc_args << ", ";
         }
+        else
+        {
+          b = true;
+        }
         proc_args << v.second;
       }
       proc_args << ")";
@@ -300,13 +331,21 @@ string Converter::to_mcrl2(OrdinaryParallel& opar, Context context)
       {
         process << proc_args.str();
       }
+      else
+      {
+        b = true;
+      }
     }
     BOOST_FOREACH(string par_subproc, par_subprocs)
     {
-      if (bb++)
+      if (bb)
       {
         process << " || ";
       }
+      else
+      {
+        bb = true;
+      }
       process << par_subproc;
     }
   }
@@ -319,10 +358,14 @@ string Converter::to_mcrl2(Send& send, Context context)
   bool b = false;
   BOOST_FOREACH(shared_ptr<Term> t, *send.terms)
   {
-    if (b++)
+    if (b)
     {
       st["terms"] << ", ";
     }
+    else
+    {
+      b = true;
+    }
     st["terms"] << to_mcrl2(t, context);
   }
 
@@ -370,19 +413,27 @@ string Converter::to_mcrl2(DY& dy, Context context)
       ds.push_back(d);
     }
 
-    if (b++)
+    if (b)
     {
       st["known_names"] << ", ";
     }
+    else
+    {
+      b = true;
+    }
     stream_cartesian_product_args(st["known_names"], ds, n.first, "", ", ");
   }
   string hint = dy.hint();
   if (!hint.empty())
   {
-    if (b++)
+    if (b)
     {
       st["known_names"] << ", ";
     }
+    else
+    {
+      b = true;
+    }
     st["known_names"] << hint;
   }
 
@@ -481,10 +532,14 @@ void Converter::to_mcrl2(PMatchTerms& pmt, Context& context, StringTemplate& st,
     bool b = false;
     BOOST_FOREACH(shared_ptr<Term> t, *pmt.match_terms)
     {
-      if (b++)
+      if (b)
       {
         st["pm_terms"] << ", ";
       }
+      else
+      {
+        b = true;
+      }
       st["pm_terms"] << to_mcrl2(t, context);
     }
   }
@@ -515,10 +570,14 @@ void Converter::to_mcrl2(PMatchTerms& pmt, Context& context, StringTemplate& st,
         if (v != context.vars.end())
         {
           v->second += "'";
-          if (b++)
+          if (b)
           {
             prime_vars << ", ";
           }
+          else
+          {
+            b = true;
+          }
           prime_vars << v->second;
         }
       }
@@ -542,10 +601,14 @@ void Converter::to_mcrl2(PMatchTerms& pmt, Context& context, StringTemplate& st,
     bool b = false;
     BOOST_FOREACH(shared_ptr<Term> t, *pmt.match_terms)
     {
-      if (b++)
+      if (b)
       {
         st["pm_terms_updated"] << ", ";
       }
+      else
+      {
+        b = true;
+      }
       st["pm_terms_updated"] << to_mcrl2(t, context);
     }
   }
@@ -570,11 +633,15 @@ void Converter::to_mcrl2(PMatchTerms& pmt, Context& context, StringTemplate& st,
       context.vars[tv->name] = tv->name;
 
       //draw separators
-      if (b++)
+      if (b)
       {
         st["vars"] << ", ";
         st["type_exprs"] << templates.fmt_string("type_sep");
       }
+      else
+      {
+        b = true;
+      }
 
       //draw type expr
       StringTemplate sf_type = (tv->type==TypedVar::V_Name) ? templates.get("type_n") : templates.get("type_c");
@@ -671,10 +738,14 @@ string Converter::to_mcrl2(Ciphertext& ct, const Context& context)
   bool b = false;
   BOOST_FOREACH(shared_ptr<Term> t, *ct.terms)
   {
-    if (b++)
+    if (b)
     {
       st["terms"] << ", ";
     }
+    else
+    {
+      b = true;
+    }
     st["terms"] << to_mcrl2(t, context);
   }
 
@@ -739,10 +810,14 @@ string Converter::to_mcrl2(Annotation& anno, const Context& context)
       //Cryptopoint_list& cps = *(anno.dest_orig);
       BOOST_FOREACH(shared_ptr<Cryptopoint> cp, *anno.dest_orig)
       {
-        if (b++)
+        if (b)
         {
           st["destorig"] << ", ";
         }
+        else
+        {
+          b = true;
+        }
         st["destorig"] << to_mcrl2(*cp);
       }
 
@@ -758,10 +833,14 @@ string Converter::to_mcrl2(Annotation& anno, const Context& context)
           pair<string, string> mv;
           BOOST_FOREACH(mv, context.mvars)
           {
-            if (b++)
+            if (b)
             {
               sf_ai["indices"] << ", ";
             }
+            else
+            {
+              b = true;
+            }
             sf_ai["indices"] << mv.first;
           }
           st["cpdy_ai"] << sf_ai.get();
@@ -804,6 +883,7 @@ string Converter::to_mcrl2(Cryptopoint& cp)
   }
 }
 
+static
 void stream_sort_def(ostringstream& structs, map<string, size_t>& m)
 {
   pair<string, size_t> p;
@@ -817,13 +897,14 @@ void stream_sort_def(ostringstream& structs, map<string, size_t>& m)
   bool b = false;
   BOOST_FOREACH(p, m)
   {
-    if (b++)
+    if (b)
     {
       structs << "  | ";
     }
     else
     {
       structs << "    ";
+      b = true;
     }
     structs << p.first;
     if (p.second>0)
